import java.awt.*;
import java.awt.event.*;
import java.awt.Font.*;
import java.util.*;
import java.lang.*;
import java.io.*;
import javax.swing.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;

// org.apache.xmlrpc (previously known as helma) and uk.co.wilson.xml are 
// available at http://ws.apache.org/xmlrpc/
import org.apache.xmlrpc.*;
import uk.co.wilson.xml.*;



public class NeosClient implements ActionListener {
  static String [] args;
  public JFrame mainFrame;
  JPanel solverPanel;
  JFrame adminFrame;
  NeosXMLParser solverParser;
  JPanel mainPanel;

  JPanel connectionPane;
  JLabel hostLabel;
  JLabel portLabel;
  JTextField hostTextField;
  JTextField portTextField;
  JButton connectButton;
  JButton submitButton;
  JTextArea messagePane;


  
  JTextField jobNumberText;
  JTextField passwordText;

  JMenuBar menuBar;
  JMenu solverMenu;
  JMenu jobsMenu;
  
  public JobTabs tabs;
  Hashtable tabindices;

  JDialog errorBox;

  Integer currentJob;
  String currentPassword;
  Hashtable categories;

  CommunicationWrapper neos;
  boolean connected;
  
  public final Font menuFont = new Font("TimesRoman", Font.BOLD, 14);
  public final Font textFont = new Font("Monospaced", Font.PLAIN, 12);


  public static void main(String[] arguments) {
    //Schedule a job for the event-dispatching thread:
    //creating and showing this application's GUI.
    args = arguments;
    javax.swing.SwingUtilities.invokeLater(new Runnable() {
	public void run() {
	  createAndShowGUI();
	}
      });
  }
  
  public static void createAndShowGUI() {     
    //Make sure we have nice window decorations.
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName()); 
    } catch (Exception e) {
      System.out.println(e.getMessage());
    }
    NeosClient client = new NeosClient("http://neos.mcs.anl.gov:3332");
  }

  /***************/
  /* Constructor */
  /***************/
  NeosClient(String url) {
  

    //Create and set up the window.
    mainFrame = new JFrame("Neos Submission Tool");
    mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    mainFrame.setSize(new Dimension(120, 40));


    //Add the widgets.
    addWidgets();
    addMenus();

    //Display the window.
    mainFrame.pack();
    mainFrame.setVisible(true);
  }


  void addWidgets() {

    //Create and set up the panels.
    JPanel hostPane = new JPanel();
    hostPane.setLayout(new GridLayout(2,2));
    hostLabel = new JLabel("Server host");
    hostTextField = new JTextField("neos.mcs.anl.gov");
    portLabel = new JLabel("Server port");
    portTextField = new JTextField("3332");
    connectButton = new JButton("Connect to Server");
    connectButton.setActionCommand("Connect to Server");
    connectButton.addActionListener(this);
    hostPane.add(hostLabel);
    hostPane.add(hostTextField);
    hostPane.add(portLabel);
    hostPane.add(portTextField);
    messagePane = new JTextArea(25,50);
    messagePane.setEditable(false);
    
    connectionPane = new JPanel();
    connectionPane.setLayout(new BoxLayout(connectionPane,BoxLayout.PAGE_AXIS));
    connectionPane.add(hostPane);
    connectionPane.add(connectButton);
    connectionPane.add(messagePane);
    
    tabs = new JobTabs(this);
    tabs.addTab("Connect",connectionPane);
    
    hostLabel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
    portLabel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

  }

  void addMenus() {
    /** Create Menubars ******************************/
    menuBar=new JMenuBar();
    mainFrame.setJMenuBar(menuBar);
    
    /* File menu */
    JMenu fileMenu=new JMenu("File");
    fileMenu.setFont(menuFont);
    JMenuItem save = new JMenuItem("Save Text");
    fileMenu.add(save);
    save.addActionListener(this);
    fileMenu.addSeparator();
    JMenuItem exit = new JMenuItem("Exit");
    fileMenu.add(exit);
    exit.addActionListener(this);
    menuBar.add(fileMenu);

    /* Options menu */
    jobsMenu = new JMenu("Jobs");
    jobsMenu.setFont(menuFont);
    JMenuItem queueMenu = new JMenuItem("Queue");
    jobsMenu.add(queueMenu);
    queueMenu.addActionListener(this);
    JMenuItem resultsMenu = new JMenuItem("Results");
    jobsMenu.add(resultsMenu);
    resultsMenu.addActionListener(this);
    JMenuItem killMenu = new JMenuItem("Kill");
    jobsMenu.add(killMenu);
    killMenu.addActionListener(this);
    jobsMenu.addActionListener(this);
    jobsMenu.setEnabled(false);
    menuBar.add(jobsMenu);
    


    /* Solvers menu */
    solverMenu = new JMenu("Solvers");
    solverMenu.setEnabled(false);
    menuBar.add(solverMenu);

    /* Help Menu */
    JMenu aboutMenu = new JMenu("Help");
    aboutMenu.setFont(menuFont);
    aboutMenu.add(new JMenuItem("Submit Client Help"));
    aboutMenu.add(new JMenuItem("Server Help"));
    aboutMenu.add(new JMenuItem("About"));
    aboutMenu.add(new JMenuItem("Comments & Questions"));
    aboutMenu.addActionListener(this);
    menuBar.add(aboutMenu);


  }

  /************************************************/
  /** Error Dialog ********************************/
  /************************************************/
  
  public void errorDialog(String errorMessage) {
    int mRows=5, mCols=35;
    errorBox = new JDialog(mainFrame, "Error", true);
    JTextArea message = new JTextArea(errorMessage, mRows, mCols);
    JScrollPane messagePane = new JScrollPane(message);
    JButton OKButton = new JButton("OK");
    OKButton.setActionCommand("close dialog box");
    OKButton.addActionListener(this);
    errorBox.getContentPane().add("Center", messagePane);
    errorBox.getContentPane().add("South", OKButton);
    errorBox.pack();
    errorBox.show();
  }

  public void actionPerformed(ActionEvent event) {
    String theSource=event.getActionCommand();
    System.out.println(theSource);
    // Handle Menu actions
    if (event.getSource() instanceof JMenuItem) {
      if (theSource.compareTo("Save Text") == 0) {
	boolean completed = false;
	//completed = saveAs(note.getText());
	if (completed==false) {
	  //flashFeedback("\n\tNo file saved \n");
	}
      } else if (theSource.compareTo("Exit") == 0) {
	mainFrame.dispose();
	System.exit(0);
      } else if (theSource.compareTo("Queue") == 0) {
	try {
	  printQueue();
	} catch (XmlRpcException e) {
	  errorDialog(e.getMessage());
	}
      } else if (theSource.compareTo("Results") == 0) {
	showAdminForm("getFinalResults");
      } else if (theSource.compareTo("Kill") == 0){
	showAdminForm("killJob");
      } else if (theSource.compareTo("About") == 0) {
	//aboutUs("Optimization Technolgy Center,\nArgonne National Lab and\nNorthwestern University,\nDecember 2004");
      } else if (theSource.compareTo("Comments & Questions") == 0) {
	//aboutUs("\nMail questions or comments to\n<neos-comments@mcs.anl.gov>\n");
      } else if (theSource.compareTo("Server Help") == 0) {
	//pass
      } else if (theSource.compareTo("Submit Client Help") == 0) {
	//pass
      } else if (theSource.compareTo("Update Solvers") == 0) {
	try {
	  createSolverMenu();
	} catch (Exception e) {
	  errorDialog("Solver Menu Build failed.\n"+e.getMessage());
	}
      } else { // Picked a Solver (theSource is "cat:solver:input")
	try {
	  String [] names = theSource.split(":");
	  buildSolverInterface(names[0],names[1],names[2]);
	} catch (XmlRpcException e) {
	  errorDialog(e.getMessage());
	} catch (NeosXMLException  e) {
	  errorDialog(e.getMessage());
	}
      }
	
		 

    } else if (event.getSource() instanceof JButton) {
      if (theSource.compareTo("Connect to Server") == 0) {
	try {
	  String neosHost = hostTextField.getText();
	  String neosPort = portTextField.getText();
	  neos = new CommunicationWrapper(new XmlRpcClient("http://"+neosHost+":"+neosPort), this);
	  connected = true;
	} catch (IOException e) { 
	  errorDialog("Connection Failed.\n" + e.getMessage());
	}
	
	try {
	  messagePane.setText((String)neos.execute("welcome",new Vector(),5000));
	  messagePane.setCaretPosition(0);
	  createSolverMenu();
	} catch (XmlRpcException e) {
	  errorDialog(e.getMessage());
	}
	
      
      } else if (theSource.compareTo("close dialog box") == 0) {
	errorBox.dispose();

      } else if (theSource.compareTo("Submit") == 0) {
	try {
	  submitToNeos();
	} catch (XmlRpcException e) {
	  errorDialog(e.getMessage());
	}
      } else if (theSource.compareTo("killJob") == 0) {
	try {
	  submitAdmin("killJob");
	} catch (XmlRpcException e) {
	  errorDialog(e.getMessage());
	}
      } else if (theSource.compareTo("getFinalResults") == 0) {
	try {
	  submitAdmin("getFinalResults");
	} catch (XmlRpcException e) {
	  errorDialog(e.getMessage());
	}
      } else if (theSource.startsWith("close tab")) {
	tabs.removeTab(theSource.substring(10));
      }

    }
  }
  void printQueue() throws XmlRpcException {
    Vector params = new Vector();
    String queue = (String)neos.execute("printQueue",params,5000);
    messagePane.setText(queue);

  }

  void createSolverMenu()  throws XmlRpcException {
    // Destroy previous solver menu
    solverMenu.removeAll();
    Vector params = new Vector();
    
    categories = (Hashtable)neos.execute("listCategories",params,5000);
    
    Iterator values = categories.values().iterator();
    params.setSize(1);
    JMenuItem menuItem;
    String solverName;
    while (values.hasNext()) {
      String cat =  (String)(values.next());
      // Ignore kestrel solvers
      if (cat.compareTo("kestrel") == 0) 
	continue;
      JMenu catMenu = new JMenu(cat);
      catMenu.setFont(menuFont);
      params.set(0,cat);
      Vector list = (Vector) neos.execute("listSolversInCategory",params,5000);
      Iterator solvers = list.iterator();
      
      while (solvers.hasNext()) {
	solverName = (String) solvers.next();
	menuItem = new JMenuItem(solverName);
	menuItem.setActionCommand(cat + ":" + solverName);
	menuItem.addActionListener(this);
	menuItem.setFont(menuFont);
	catMenu.add(menuItem);
      }
      solverMenu.add(catMenu);
    }
    solverMenu.setEnabled(true);
    jobsMenu.setEnabled(true);
  }

  public void buildSolverInterface(String category, String solver, 
		String inputMethod) throws XmlRpcException, NeosXMLException {

    // Get the solver xml string
    Vector params = new Vector();
    params.add(category);
    params.add(solver);
    params.add(inputMethod);

    String xmlstring = (String) neos.execute("getXML",params,5000);
    
    solverParser = new NeosXMLParser();
    solverParser.parse(xmlstring);
    
    JPanel solverTab = solverParser.formGUI(this);
    tabs.addTab(solver,solverTab);

  }

 
	       
 
  void submitToNeos() throws XmlRpcException {
    String xmldoc = solverParser.writeDocument();
    Vector params = new Vector();
    params.add(xmldoc);
    params.add("JAVA user");
    params.add("JAVA Submission Tool");
    String buffer;
    
    Vector v = (Vector)neos.execute("submitJob",params);
    
    currentJob = (Integer)v.get(0);
    currentPassword = (String)v.get(1);
    
    tabs.addJob(currentJob);
    tabs.setBusy(currentJob);
    tabs.setText(currentJob,"Job #"+currentJob.toString() + "submitted to NEOS.\n");

    params.clear();
    buffer = (String)neos.execute("printQueue",params);
    tabs.appendText(currentJob, buffer);
    
    ResultsReceiver receiver = new ResultsReceiver(this,currentJob,currentPassword);
    receiver.start();

      
  }



  /* 
     Runs either killJob or getFinalResults
  */
  public void showAdminForm(String adminJob) {
    adminFrame = new JFrame(adminJob);
    JPanel adminPane = new JPanel();
    
    jobNumberText  = new JTextField(10);
    JLabel jobNumberLabel = new JLabel("Enter Job Number");
    jobNumberLabel.setLabelFor(jobNumberText);

    passwordText = new JTextField(10);
    JLabel passwordLabel = new JLabel("Enter Password");
    passwordLabel.setLabelFor(passwordText);

    adminPane.add(jobNumberLabel);
    adminPane.add(jobNumberText);
    adminPane.add(passwordLabel);
    adminPane.add(passwordText);
      
    JButton submitButton = new JButton(adminJob);
    submitButton.setActionCommand(adminJob);
    submitButton.addActionListener(this);
    adminPane.add(submitButton);
    
    adminFrame.getContentPane().add(adminPane);
    adminFrame.pack();
    adminFrame.setVisible(true);
  }

  public void submitAdmin(String adminJob) throws XmlRpcException {
    Vector params = new Vector();
    System.out.println(jobNumberText.getText());
    try {
      params.add(new Integer(jobNumberText.getText()));
    } catch (NumberFormatException e) {
      errorDialog(e.getMessage());
      return;
    }
    System.out.println((Integer)params.firstElement());
    params.add((Object)passwordText.getText());
    Object  result = neos.execute(adminJob,params);
    tabs.addJob(adminJob);
    if (result instanceof String) {
      tabs.setText(new Integer(0),(String)result);
    } else if (result instanceof byte[]) {
      tabs.setText(new Integer(0),new String((byte[]) result));
    } else
      throw new XmlRpcException (0,"Unkown response from NEOS");
    adminFrame.dispose();
  }
}

class ResultsReceiver extends Thread {
  Integer jobNumber;
  String password;
  NeosClient parent;
  public ResultsReceiver(NeosClient p,Integer job, String pass) {
    this.parent = p;
    this.jobNumber = job;
    this.password = pass;
  }
  public void run()  {
    Vector params = new Vector();
    
    params.add(this.jobNumber);
    params.add(this.password);

    
    Vector offsetParams = new Vector();
    offsetParams.add(this.jobNumber);
    offsetParams.add(this.password);
    int offset = 0;
    offsetParams.add(new Integer(offset));
    String status;
    
    try {
      status = (String)parent.neos.execute("getJobStatus",params);
      while (status.compareTo("Done") != 0) {
	Vector retval = (Vector)parent.neos.execute("getIntermediateResults",offsetParams);
	offsetParams.set(2,(Integer)retval.get(1));
	Object output = (Object)retval.get(0);
	String newText;
	if (output instanceof String) 
	  newText = (String)output;
	else 
	  newText = new String((byte[]) output);
	parent.tabs.appendText(jobNumber,newText);
	System.out.println("getting Job Status");
	status = (String)parent.neos.execute("getJobStatus",params);
      }
      System.out.println("getting Final results.");
      Object retval = parent.neos.execute("getFinalResults",params);
      parent.tabs.setDone(jobNumber);
      if (retval instanceof String) 
	parent.tabs.setText(jobNumber,(String)retval);
      else if (retval instanceof byte[])
	parent.tabs.setText(jobNumber,new String((byte[]) retval));
    } catch (XmlRpcException  e) {
      parent.errorDialog(e.getMessage());
    }
  }
} 
  
class CommunicationWrapper implements AsyncCallback{
  boolean connected;
  boolean waiting;
  Object response;
  NeosClient parent;
  XmlRpcClient server;
  Exception exception;
  Date startTime;
  Stack stack;
  final static int sleepInterval = 100; //milliseconds
  
  public CommunicationWrapper(XmlRpcClient s, NeosClient p) {
    server = s;
    parent = p;
    connected = false;
  }

  public Object execute(String method, Vector params) throws XmlRpcException {
    try {
      return server.execute(method,params);
    } catch ( IOException e) {
      throw new XmlRpcException(0, e.getMessage());
    }
  }
  
  
  public Object execute(String method, Vector params, long timeout)  throws XmlRpcException{
    if (timeout < 0) {
      timeout = 1000000000L; // ~300 hrs
    }
    waiting = true;
    Cursor oldCursor = parent.mainFrame.getContentPane().getCursor();
    parent.mainFrame.getContentPane().setCursor(new Cursor(Cursor.WAIT_CURSOR));
    response = null;
    exception = null;
    startTime=new Date();
    server.executeAsync(method,params,this);
    while (waiting && (new Date().getTime()-startTime.getTime() < timeout)) {
      try {Thread.sleep(sleepInterval);} catch (InterruptedException e) {}
    }      
    parent.mainFrame.getContentPane().setCursor(oldCursor);
    if (exception != null) 
      throw new XmlRpcException(0,exception.getMessage());
    if (response == null) {
      throw new XmlRpcException(0,"Error communicating with server.");
    }
    return response;
  }

  public void handleError(java.lang.Exception e, java.net.URL url, java.lang.String method)  {
    System.out.println("Error occured");
    if (waiting) {
      waiting = false;
      exception = e;
      response = null;
    }
  }
  
  public void handleResult(java.lang.Object result, java.net.URL url, java.lang.String method)  {
    System.out.println("Normal execution");
    if (waiting) {
      waiting = false;
      response = result;
    }
  }

}

class JobTabs  {
  NeosClient parent;
  JTabbedPane tabPane;
  Hashtable outputPanes; // indexed by jobnumber 
  Hashtable oldCursors; // indexed by jobnumber
  public JobTabs(NeosClient p) {
    parent = p;
    tabPane = new JTabbedPane();
    parent.mainFrame.getContentPane().add(tabPane);
    outputPanes = new Hashtable();
    oldCursors = new Hashtable();
  }
  
  public void addTab(String name, JComponent c) {
    JPanel panel = new JPanel(new BorderLayout());
    if (name.compareTo("Connect") != 0) {
      JButton closeMe = new JButton("Close Tab");
      closeMe.setActionCommand("close tab " + name);
      closeMe.addActionListener(parent);
      panel.add(closeMe,BorderLayout.PAGE_START);
    }
    JScrollPane sp = new JScrollPane(c);
    panel.add(sp);
    tabPane.add(name,panel);
    tabPane.setSelectedComponent(panel);
  }
  public void addJob(Integer jobNumber) {
    JPanel panel = new JPanel(new BorderLayout());
    JButton closeMe = new JButton("Close Tab");
    closeMe.setActionCommand("close tab job #" + jobNumber.toString());
    closeMe.addActionListener(parent);
    
    JTextArea textArea = new JTextArea(30,50);
    textArea.setEditable(false);
    outputPanes.put(jobNumber,textArea);
    panel.add(closeMe,BorderLayout.PAGE_START);
    JScrollPane scroller = new JScrollPane(textArea);
    panel.add(scroller,BorderLayout.CENTER);
    tabPane.add("job #" + jobNumber.toString(),panel);
    tabPane.setSelectedComponent(panel);
  }
  public void addJob(String s) {
    
  }
  public void setText(Integer jobNumber, String msg) {
    JTextArea textArea = (JTextArea)outputPanes.get(jobNumber);
    textArea.setText(msg);
  }
  public void appendText(Integer jobNumber, String msg) {
    JTextArea textArea = (JTextArea)outputPanes.get(jobNumber);
    textArea.setText(textArea.getText() + msg);
  }
  public void setBusy(Integer jobNumber) {
    JTextArea textArea = (JTextArea)outputPanes.get(jobNumber);
    oldCursors.put(jobNumber, textArea.getCursor());
    textArea.setCursor(new Cursor(Cursor.WAIT_CURSOR));
  }
  public void setDone(Integer jobNumber) {
    JTextArea textArea = (JTextArea)outputPanes.get(jobNumber);
    textArea.setCursor((Cursor)oldCursors.get(jobNumber));
  }
  public void removeTab(String tabName) {
    int tabIndex = tabPane.indexOfTab(tabName);
    if (tabIndex >= 0) 
      tabPane.remove(tabIndex);
  }
  public void saveTab(String tabName) {
    
  }
}
      
    
    
  
    
