#!/usr/bin/env python
import os
import re
import xmlrpclib
import sys
import time
import socket
import uu
import StringIO

neos = xmlrpclib.Server("http://neos.mcs.anl.gov:3332")
allKestrelSolvers = neos.listSolversInCategory("kestrel")
kestrelAmplSolvers = []
for s in allKestrelSolvers:
  i = s.find(':AMPL')
  if i > 0:
    kestrelAmplSolvers.append(s[0:i])


options = os.getenv("kestrel_options")
if options is not None: 
  m = re.match('solver=(\S+)$',options)
  if m:
    solver_name=m.groups()[0]
    if solver_name not in kestrelAmplSolvers:
      sys.stdout.write("%s is not available on NEOS.  Choose from:\n" % solver_name)
      for s in kestrelAmplSolvers:
        sys.stdout.write("\t%s\n"%s)
      sys.stdout.write('To choose: option kestrel_options "solver=xxx"\n\n')
      sys.exit(0)
      
if options is None or m is None:
  sys.stdout.write("No solver name selected.  Choose from:\n")
  for s in kestrelAmplSolvers:
    sys.stdout.write("\t%s\n"%s)
  sys.stdout.write('\nTo choose: option kestrel_options "solver=xxx"\n\n')
  sys.exit(0)


nlfile = open(sys.argv[1]+".nl","rb")
uustring = StringIO.StringIO()
uu.encode(nlfile,uustring)
nlfile.close()
     
ampl_files={}
for key in ['adj','col','env','fix','spc','row','slc','unv']:
  if os.access(sys.argv[1]+"."+key,os.R_OK):
    f = open(sys.argv[1]+"." +key,"r")
    val=""
    buf = f.read()
    while buf:
      val += buf
      buf=f.read()
    f.close()
    ampl_files[key] = val


  
# Add any AMPL-created environment variables to dictionary
solver_options = "kestrel_options:solver=%s\n" % solver_name
solver_options += "myopts:ddd\n"
solver_options_key = "%s_options" % solver_name
if solver_options_key in os.environ.keys():
  solver_options+="%s:%s\\n"%(solver_options_key, os.getenv(solver_options_key))
# mip, aux, ...

xml = """ 
      <document>
      <category>kestrel</category>
      <solver>%s</solver>
      <inputType>AMPL</inputType>
      <solver_options>%s</solver_options>
      <nlfile><![CDATA[%s]]></nlfile>\n""" % (solver_name,solver_options,
                                              uustring.getvalue())

for key in ampl_files.keys():
  xml += "<%s>%s</%s>\n" % (key,ampl_files[key],key)

xml += "</document>"

user = "%s on %s" % (os.getenv('LOGNAME'),socket.getfqdn(socket.gethostname()))
(jobNumber,password) =neos.submitJob(xml,user,"KESTREL")


status = "Waiting"
offset = 0
time.sleep(1)
while (status == "Waiting" or status=="Running"):
  (results,offset) = neos.getIntermediateResults(jobNumber, password,offset)
  results = results.data
  if len(results):
    print results
  status = neos.getJobStatus(jobNumber,password)

print "status=done"
results = neos.getFinalResults(jobNumber,password)
if isinstance(results,xmlrpclib.Binary):
  resultsuu = StringIO.StringIO(results.data)
else:
  resultsuu=StringIO.StringIO(results)

#uu-decode results to kestrel.sol
# Well try to anyway, any errors will result in error strings in .sol file
#  instead of solution.
tmpfile = open("results.tmp","w")
tmpfile.write(results.data)
tmpfile.close()
solfile = open(sys.argv[1] + ".sol","wb")
try:
  uu.decode(resultsuu,solfile)
except:
  pass







