#!/usr/bin/env python -u
#  the -u is for unbuffered input/output, and default binary file reads

import sys
import os
import cgi
import time
import string
import signal
import select
import socket
import xmlrpclib
import base64
import xml.dom.minidom

import NeosXML
import NeosServer
import WebPrinter
from NEOS.config import Variables


class CGIPrinter:

  def __init__(self):
    self.neos = xmlrpclib.Server("http://%s:%d"
                                 %(Variables.NEOS_HOST,Variables.NEOS_PORT))
    #Test connection
    self.status=""
    signal.signal(signal.SIGALRM,self.alarmhandler)
    signal.alarm(10)
    try:
      self.neos.ping()
    except Exception, (err):
      self.status="No connection"
    signal.alarm(0)

    self.form = cgi.FieldStorage()
    self.admin = self.form.getfirst("admin")
    if self.admin is None:
      self.solver = self.form.getfirst("solver")
      self.category = self.form.getfirst("category")
      self.inputMethod = self.form.getfirst("inputMethod")
      if self.form.getfirst("priority") == "short":
        self.priority = "short"
      else:
        self.priority = "long"
      self.email = self.form.getfirst("email")
      if self.email is None:
        self.email=""
      self.token = self.category + ":" + self.solver
    
  def printPage(self):
    self.printHeader()

    if self.status:
      sys.stdout.write("Could not connect to NEOS Server.\n")
      
    elif self.admin is None:

      # If it's an example, then jobin is just the example .txt file
      if self.form.has_key("example"):
        interface="Web Example"
        exampleName = self.form.getfirst("example")
        jobinFile = open(os.path.join(Variables.NEOS_WWW_HOME,"solvers",
                                      self.token, exampleName+".txt"),"r")
        self.jobIn =jobinFile.read().replace('</inputType>','</inputType>\n<client>%s@%s</client><email>%s</email>' % (os.getenv('HTTP_USER_AGENT'),os.getenv('REMOTE_HOST'),self.email))
      else:
        # Otherwise, look up solver xml document
        interface = "Web"
        self.solverdoc = NeosXML.SolverParser(self.neos.getXML(self.category,self.solver,self.inputMethod))

        self.createJobIn()



      # Submit job to NEOS
      (jobNumber,password) = self.neos.submitJob(self.jobIn,"WEBUSER@"+os.getenv('REMOTE_HOST'),interface)
      if jobNumber == 0:
        raise Exception, ("Error in submission xml file.")

      sys.stdout.write(self.neos.printQueue())
      sys.stdout.write("jobnumber=%d\tpassword=%s\n" % (jobNumber,password))
      sys.stdout.flush()



      signal.signal(signal.SIGALRM,self.alarmhandler)
      signal.alarm(60)
      msg = ""
      self.status = "Running"
      offset=0
      while self.status == "Running" or self.status == "Waiting":
        try:
          (msg,offset)=self.neos.getIntermediateResults(jobNumber,password,offset)
          if len(msg.data):
            sys.stdout.write(msg.data)
            sys.stdout.flush()

          if self.neos.getJobStatus(jobNumber,password) == "Done":
            self.status = 'Done'
        except socket.error, (errno, msg):
          self.status = "Timer"
          
        except Exception, (e):
          self.status = "Error"
          sys.stdout.write("exception! "+str(e))
          sys.stdout.write('Get results from <a href=%s/jobs/%d.html">%s/jobs/%d.html</a>\n' % (Variables.NEOS_WEB_SITE,jobNumber,Variables.NEOS_WEB_SITE,jobNumber))
          sys.stdout.flush()
        


      if self.status=="Timer":
        sys.stdout.write("No results after 1 minute, stopping.\n")
        sys.stdout.write("Get results from this nonexistant link.\n")
      else:
        sys.stdout.write("Exiting normally...\n")
      
      time.sleep(5)
      sys.stdout.write('<META HTTP-EQUIV="Refresh" CONTENT="3; URL=%s/jobs/%d.html">\n' %  (Variables.NEOS_WEB_SITE,jobNumber))


    else:
      # Administration job
      WebPrinter.printHtmlHeader(sys.stdout,None,None,None,None)
      if self.admin=="queue":
        sys.stdout.write(self.neos.printQueue())
        return

      #Check for valid jobnumber, password entries
      try:
        jobNumber = int(self.form.getfirst("jobnumber"))
      except TypeError, e:
        sys.stdout.write("Error, no integer value given for job number.\n")
        return
      try:
        password = self.form.getfirst("pass")
      except TypeError, e:
        sys.stdout.write("Error, no value given for password.\n")
        return

      if self.admin=="results":
        sys.stdout.write(self.viewJob(jobNumber,password))
      else:
        sys.stdout.write(self.neos.killJob(jobNumber,password))

      self.printFooter()

  def htmlformat(self,s):
    return s.replace('&','&amp;').replace('<','&lt;').replace('>','&gt;')
  def alarmhandler(self,signo,frame):
    self.status = "Timer"
      


  def createJobIn(self):
    self.jobIn="<document>\n<category>%s</category>\n<solver>%s</solver>\n"\
                % (self.category, self.solver)
    self.jobIn+="<inputType>%s</inputType>\n" % (self.inputMethod)
    self.jobIn+="<client>%s@%s</client>\n" % (os.getenv('HTTP_CLIENT_AGENT'),
                                              os.getenv('REMOTE_HOST'))

    self.jobIn+="<priority>%s</priority>\n" % self.priority
    if self.email is not None:
      self.jobIn+="<email>%s</email>\n" % self.email
      
    for i in range(0,len(self.solverdoc.items)):
      if not self.form.has_key("field.%d" % (i+1)):
        continue
      data = self.form["field.%d"%(i+1)]

      # if multiple entries with this name given (should never happen, but
      # who knows what users can do...) pick first one
      if isinstance(data,list):
        data = data[0]

      # read in file
      if data.file:
        value=""
        while 1:
          chunk = data.file.read(100000)
          if chunk:
            value += chunk
            if len(value) > Variables.MAX_FILE_LENGTH:
              sys.stdout.write("Error, files must be smaller than %d bytes.\n"
                               % Variables.MAX_FILE_LENGTH)
              self.printFooter()
              sys.exit(1)
              
          else:
            break

        # handle binary files
        # Check for non-printable characters (not foolproof!)
        testlength = 1000
        if len(value) < 1000:
          testlength = len(value)
        printable=1
        for c in value[0:testlength-1]:
          if c not in string.printable:
            printable = 0
            break
        if not printable:
          value = "<base64>%s</base64>\n" % base64.encodestring(value)
          
      else:
        value = data


      self.jobIn += "<%s><![CDATA[%s]]></%s>\n\n" %\
                      (self.solverdoc.items[i].getToken(),
                       value,self.solverdoc.items[i].getToken())

    self.jobIn += "</document>\n"


  def printHeader(self):
    sys.stdout.write('''
%s 200 OK\n\
Server: %s\n\
Content-type: text/html\n\n\
<HTML>\n<BODY BGCOLOR="%s">\n<pre>\n\
''' % (os.getenv('SERVER_PROTOCOL'),os.getenv('SERVER_SOFTWARE'),Variables.NEOS_WEB_BGCOLOR))
      
    sys.stdout.flush()

    

  def printFooter(self):
    sys.stdout.write("\n</PRE>\n</BODY></HTML>\n")
    

  def viewJob(self,jobNumber,password):

    status = self.neos.getJobStatus(jobNumber,password)
    if status == "Unknown Job":
      msg = "Job %d is unknown. Check that you entered the correct job #" % jobNumber
      
    elif status == "Waiting":
      msg =  "Job %d is waiting in queue.\n%s"%(jobNumber,self.neos.printQueue())

    elif status == "Running":
      msg = "Job %d is running.\n Up-to-date Results:\n%s" % (jobNumber,self.neos.getFinalResults(jobNumber,password))

    else:
      results =self.neos.getFinalResults(jobNumber,password)
      if isinstance(results,xmlrpclib.Binary):
        results = results.data
      
      msg ="Job %d has finished.\n%s" % (jobNumber,results)

    return msg
    
      
    
    
