#!/usr/bin/env python
import sys
import os
import signal
import re
import poplib
import email
import email.MIMEText
import smtplib
import xml.parsers.expat
import xml.dom.minidom
import logging
import logging.handlers
import xmlrpclib
import time
import NeosTools

from config import Variables

global logger

class MailHandler:
  def __init__(self):
    self.neos = xmlrpclib.Server("http://%s:%d" % (Variables.NEOS_HOST,
                                                   Variables.NEOS_PORT))


  def runForever(self):
    # Enter into PROCS table
    
    start_time = time.strftime("%Y-%m-%d %H:%M:%S")
    db = NeosTools.NeosDB()
    db.query('REPLACE INTO PROCS (name,pid,start_time) VALUES("MailHandler","%d","%s")' % (os.getpid(),start_time))
    db.close()

    while 1:
      msglist = []
      logger.debug("Checking for new mail...")
      try:
        #Open the mailbox and check for any new mail submissions.
        pop = poplib.POP3(Variables.POP_SERVER)
        pop.rpop(Variables.POP_USERNAME)
        msglist.extend(pop.list()[1])
      except Exception, (e):
        logger.exception(e)

      logger.debug("messages: " + str(msglist))
      for item in msglist:
        msgDeleted = False
        try:
          job = ""
          m = re.match(r'(\d+) (\d+)',item)
          msgNumber = long(m.groups()[0])
          msgLength = long(m.groups()[1])
          emsg = "\n".join(pop.retr(msgNumber)[1])

          # Read email message
          msg = email.message_from_string(emsg)

          # Get reply address
          sender = msg['To']
          if msg.has_key('Reply-To'):
            user = msg['Reply-To']
          elif msg.has_key('From'):
            user = msg['From']
          else:
            user = "unknownmailer"
          logger.debug("Msg received from " + user)
          if user.find( "root@mcs.anl.gov") >= 0:
            logger.warning("Received message from root@mcs.anl.gov:")
            logger.warning(str(msg))
            pop.dele(msgNumber)
            continue
            
          # If MIME format, get first xml-parsable text/plain or
          # application/octet-stream section
          jobNumber = None
          reply = ""
          for part in msg.walk():
            if part.is_multipart():
              continue

            elif part.get_content_type() == 'text/plain':
              payload = part.get_payload()
              if self.checkXML(payload):
                (jobNumber,password)=self.neos.submitJob(payload,user,'email')
                reply = "NEOS has received and assigned your submission as "
                reply +="job #%d, password '%s'\n" % (jobNumber,password)
                reply += "Current queue: \n %s\n" % self.neos.printQueue()
                break
              else:
                reply=self.checkNeosCommand(payload)
                if reply:
                  break
                
            elif part.get_content_type() == 'application/octet-stream':
              payload = part.get_payload(decode=1)
              if self.checkXML(payload):
                (jobNumber,password)=self.neos.submitJob(payload,user,'email')
                reply = "NEOS has received and assigned your submission as "
                reply +="job #%d, password '%s'\n" % (jobNumber,password)
                reply += "Current queue: \n %s\n" % self.neos.printQueue()
                break
              else:
                reply=self.checkNeosCommand(payload)
                if reply:
                  logger.debug("reply=" + reply)
                  break

          if not reply:
            logger.warning("Mail message not parseable")
            reply = "NEOS was unable to recognize your email submission."
            
          msg = email.MIMEText.MIMEText(reply)
          if jobNumber:
            msg['Subject'] = "NEOS Confirmation for Job #%d" % jobNumber
          else:
            msg['Subject'] = "Automated response from NEOS"
          msg['From'] = sender
          msg['To'] = user

          s = smtplib.SMTP()
          s.connect(Variables.POP_SERVER)
          s.sendmail(sender,user,msg.as_string())
          s.close()

          # Delete mail message
          pop.dele(msgNumber)
          msgDeleted = True

        except Exception, (e):
          logger.exception(e)

        if not msgDeleted:
          try:
            logger.warning("Could not deal with mail message. deleting")
            if msgLength < 10000:
              logger.debug(msg.as_string())
            pop.dele(msgNumber)
          except Exception, (e):
            logger.exception(e)
        

      try:
        pop.quit()
      except:
        pass

      # Check for any done jobs requiring email forwarding
      self.mailJobResults()

      time.sleep(30)


  def checkXML(self,xmlstring):
    try:
      xml.dom.minidom.parseString(xmlstring)
      return xmlstring
    except xml.parsers.expat.ExpatError, (e):
      if str(e).find(xml.parsers.expat.errors.XML_ERROR_JUNK_AFTER_DOC_ELEMENT) < 0:
        logger.exception(e)
        return None

    # try stripping xmlstring (get everything before last '>')
    m = re.match(r'^([\s\S*])[^>]*$',xmlstring)
    if m and m.groups():
      xmlstring = m.groups()[0]
    try:
      xml.dom.minidom.parseString(xmlstring)
      return xmlstring
    except xml.parsers.expat.ExpatError, (e):
      return None

    return None

  def checkNeosCommand(self,command):
    """
    If email is a Neos command (help, printQueue, getResults, etc)
    then parse and send command
    """
    logger.debug("Checking " + command)
    if command.find("help")==0:
      m = re.match(r'help\s+(\S+):(\S+):(\S+)',command)
      if m and m.groups() and len(m.groups()):
        category =m.groups()[0]
        solver = m.groups()[1]
        inputMethod = m.groups()[2]

        return self.neos.getSolverTemplate(category,solver,inputMethod)
      else:
        return self.neos.emailHelp()
    elif command.find("get queue") == 0:
      return self.neos.printQueue()
    elif command.find('get')==0:
      m = re.match(r'get results\s+(\d+)\s+(\S+)')
      if m and m.groups() and len(m.groups()):
        jobNumber = m.groups()[0]
        password = m.groups()[1]
        status = self.neos.getJobStatus(jobNumber,password)
        if status != "Done":
          return "Status of job #%d: %s" % (jobNumber,status)
        else:
          results =  self.neos.getFinalResults(jobNumber,status)
          if isinstance(results,xmlrpclib.Binary):
            results = results.data
          return results
    return None
          
          
        

    
  def mailJobResults(self):
    db = NeosTools.NeosDB(logger)
    results = db.queryAll('SELECT EMAIL.job_number, EMAIL.address, EMAIL.message, JOBS.password from EMAIL LEFT JOIN JOBS ON EMAIL.job_number=JOBS.number WHERE JOBS.status="Done" ORDER BY EMAIL.job_number LIMIT 5')
    if len(results):
      s = smtplib.SMTP()
      s.connect(Variables.POP_SERVER)
      sender = Variables.NEOS_MAIL
      for result in results:
        jobNumber = result[0]
        user = result[1]
        message = result[2]
        password = result[3]
        if message is not None:
          jobResults = message
        else:
          jobResults = db.getJobResults(jobNumber)
          if len(jobResults) > Variables.MAX_MAIL_LENGTH:
            jobResults = "Job results too long for email.  Download results from %s/admin.html\nJob Number=%d, Password=%s" % (Variables.NEOS_WEB_SITE,jobNumber,password)


        db.query('DELETE FROM EMAIL WHERE job_number=%d' % jobNumber)


        # Create mail message
        msg = email.MIMEText.MIMEText(jobResults)
        msg['Subject'] = "NEOS Results for Job #%d" % jobNumber
        msg['From'] = sender
        msg['To'] = user

        try:
          s.sendmail(sender,user,msg.as_string())
        except Exception, e:
          logger.exception(e)
      s.close()
    db.close()

  def isRunning(self):
    db = NeosTools.NeosDB()
    result = db.query('SELECT pid,start_time from PROCS WHERE name="MailHandler"')
    db.close()
    pid = 0
    start_time=""
    if result:
      pid = int(result[0])
      start_time = result[1]
      logger.info("MailHandler pid=%d started %s",pid,start_time)

      #Check to see if pid is good
      try:
        os.kill(pid,0)
        logger.warning("Process #%d is running.",pid)

      except OSError, (errno, msg):
        pid = 0

    return (pid,start_time)
            
if __name__=="__main__":
  if not os.access(Variables.LOGDIR, os.X_OK):
    os.makedirs(Variables.LOGDIR)
  logger = logging.getLogger('MailHandler')
  hdlr = logging.handlers.RotatingFileHandler(os.path.join(Variables.LOGDIR,"MailHandler"),maxBytes=100000,backupCount=1)
  formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
  hdlr.setFormatter(formatter)
  logger.addHandler(hdlr)
  logger.setLevel(logging.DEBUG)

  mailer = MailHandler()  
  # Check to see if mailer already exists
  (pid,startdate) = mailer.isRunning()
  if pid:
    sys.stdout.write("MailHandler is already running pid=%d, started %s.\n" %
                     (pid,startdate))
    sys.stdout.write("Restart? (y/n) [y]: ")

    line = sys.stdin.readline()
    if line.find('n') == 0:
      sys.exit(1)
    else:
      os.kill(pid,signal.SIGTERM)

  logger.info("Started mail handler")

  # daemonize
  pid = os.fork()
  if pid:
    os._exit(0)

  os.setsid() # Set session ID so that we have no controlling terminal
  os.umask(077) # fix creation mask
  for i in range(3):
    try:
      os.close(i)
    except OSError, e:
      if e.errno != errno.EBADF:
        raise RuntimeError('Could not close default descriptor ' + str(i))

  try:
    mailer.runForever()
  except Exception, (e):
    logger.exception(e)

  
