#!/usr/bin/env python
import MySQLdb
import xmlrpclib
import time
import socket
import sys
import logging
import logging.handlers
import os
import sys

from config import Variables
import NeosTools

global logger
  

class NeosTesting:
  def __init__(self):
    if not os.access(Variables.TESTDIR, os.X_OK):
      os.makedirs(Variables.TESTDIR)
      
  
  def getExampleList(self):
    db = NeosTools.NeosDB()
    examples = db.queryAll('SELECT xml from EXAMPLES ORDER BY solver_cat')
    return examples[0:1]
  


if __name__=="__main__":
  if not os.access(Variables.LOGDIR, os.X_OK):
    os.makedirs(Variables.LOGDIR)
  logger = logging.getLogger('tester')
  hdlr = logging.handlers.RotatingFileHandler(os.path.join(Variables.LOGDIR,"tester"),maxBytes=100000,backupCount=5)
  formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
  hdlr.setFormatter(formatter)
  logger.addHandler(hdlr)
  logger.setLevel(logging.DEBUG)


  neos = xmlrpclib.ServerProxy("http://%s:%d" % (Variables.NEOS_HOST,
                                                 Variables.NEOS_PORT))
  # Make sure that NEOS server is running
  try:
    timeout = socket.getdefaulttimeout()
    socket.setdefaulttimeout(5)
    reply = neos.ping()
    if reply.find('alive') < 0:
      logger.warning('http://%s:%d unavailable.' % (Variables.NEOS_HOST,
                                                       Variables.NEOS_PORT))
      sys.exit(1)
  except Exception, (e):
    logger.exception(e)
    sys.exit(1)

  tester = NeosTesting()
  exampleList = tester.getExampleList()
  for ex in exampleList:
    filename = ex[0]
    try:
      f = open(filename,'r')
      xml = f.read()
      (jobNumber,passwd)=neos.submitJob(xml,'testneos','TESTER')
      logger.info("(%d:%s) submitting %s to neos\n" % (jobNumber,passwd,filename))
      time.sleep(5)
    except Exception, (e):
      logger.exception(e)
        

