import re
import os
import stat
import threading
import signal
import time
import MySQLdb
import _mysql_exceptions
import xml.dom.minidom
import random
import select
import string
import xmlrpclib
import socket
import xml.parsers.expat
import tempfile
import email.MIMEText
import smtplib
import StringIO
import shutil
import popen2

import NeosXML
import WebPrinter
from config import Variables

# Used so if statements aren't needed for logging
class EmptyLogger:
  def info(self,msg):
    pass
  def debug(self,msg):
    pass
  def exception(self,msg):
    pass
  def warning(self,msg):
    pass
  def critical(self,msg):
    pass
  def error(self,msg):
    pass

class NeosCondition:
  def __init__(self):
    
    self.bigLock = threading.Lock()
    self.locks = {'job_out':{}, 'job_results':{}}

    
  def acquire(self,column,jobNumber):
    self.bigLock.acquire()
    if self.locks[column].has_key(jobNumber):
      self.locks[column][jobNumber]['refCount'] += 1
      self.bigLock.release()
      self.locks[column][jobNumber]['condition'].acquire()
    else:
      self.locks[column][jobNumber] = {'refCount':1,
                                  'condition':threading.Condition()}
      self.bigLock.release()
      self.locks[column][jobNumber]['condition'].acquire()


  def release(self,column,jobNumber):
    self.bigLock.acquire()
    if self.locks[column].has_key(jobNumber):
      self.locks[column][jobNumber]['refCount'] -= 1
      self.locks[column][jobNumber]['condition'].release()
      if self.locks[column][jobNumber]['refCount'] == 0:
        self.locks[column].pop(jobNumber)
    self.bigLock.release()

  def wait(self,column,jobNumber):
    if self.locks[column].has_key(jobNumber):
      self.locks[column][jobNumber]['condition'].wait()

  def notifyAll(self,column,jobNumber):
    if self.locks[column].has_key(jobNumber):
      self.locks[column][jobNumber]['condition'].notifyAll()


class NeosJob:
  locks = NeosCondition()
  
  def __init__(self, num=None, passwd=None, log=None):
    self.jobNumber = num
    self.password = passwd
    if passwd=="test":
      self.jobsTable = "TESTJOBS"
    else:
      self.jobsTable = "JOBS"
    
    if log:
      self.logger = log
    else:
      self.logger = EmptyLogger()

  def registerPid(self,db,pid):
    db.query('UPDATE QUEUE SET pid=%d WHERE job_number=%d' %
             (pid,self.jobNumber))


  def kill(self,db):
    status = self.getJobStatus(db)
    if status == "Bad Password":
      return "%s is wrong password for job %d" % (self.password,self.jobNumber)

    if status == "Unknown Job":
      return "Job %d does not exist." % self.jobNumber

    if status == "Done":
      return "Job #%d is finished" % self.jobNumber

    if status == "Waiting":
      db.query('REMOVE FROM QUEUE WHERE number=%d' % self.jobNumber)
      return "Job %d removed from queue." % self.jobNumber

    if status == "Running":
      # Get station job is running on
      result = db.query('SELECT station_id,pid from QUEUE where job_number=%d' % self.jobNumber)
      if result:
        stationID = result[0]
        if result[1] is None:
          return "Could not kill job #%d, Unknown pid."

        pid = int(result[1])
      else:
        return("Error finding job in database.  Contact NEOS administrator.")

      result = db.query('SELECT  host_name, port FROM STATIONS WHERE ID=%d' % stationID)
      if result:
        host_name=result[0]
        port=int(result[1])

        remote = xmlrpclib.Server('http://%s:%d' % (host_name,port))
        try:
          msg = remote.ping()
          self.logger.debug("Sending kill job#%d (pid=%d) request to %s" % (self.jobNumber,pid,host_name))
          msg = remote.killJob(self.jobNumber,self.password,pid)

          # if station doesn't recognize pid, give termination signal to NEOS
          if msg.find('Not Running'):
            msg = "Station %s cannot confirm job #%d is running.  Terminating." %\
                  (host_name,self.jobNumber)
            
            neos = xmlrpclib.Server('http://%s:%d' % (Variables.NEOS_HOST,Variables.NEOS_PORT))
            neos.submitErrorResults(self.jobNumber,self.password,"Job killed by user request -- %s did not recognize pid." % host_name)
            
          return "Kill Job #%d Request sent to %s:\n%s\n" % \
               (self.jobNumber,host_name,msg) 
          
        except socket.error, (e):
          # Can't contact solver host. Dequeue
          neos = xmlrpclib.Server('http://%s:%d' % (Variables.NEOS_HOST,Variables.NEOS_PORT))
          neos.submitErrorResults(self.jobNumber,self.password,"Job killed by user request.")
          return "Job %d removed from queue, may be zombie." % self.jobNumber
          
      else:
        return ("Error finding station ID=%d in database.  Contact NEOS administrator.\n" % stationID)
    
    else:
      return "Unknown"
    
  def submit(self,db,xmlstring,user,interface,station_id=0):
    msg = ""
    clienthost="Unknown"
    if user is None:
      user = "Unknown"
    elif interface=="email":
      clienthost=user
    else:
      m = re.match(r'(\d+\.\d+\.\d+\.\d+)',user)
      if m and m.groups() and len(m.groups()):
        clientip = m.groups()[0]
        try:
          res = socket.gethostbyaddr(clientip)
          if res and len(res):
            clienthost = res[0]
        except Exception, (e):
          self.logger.debug("gethostbyaddr: " + str(e))
    if interface is None:
      interface = "Unknown"
      
    if interface=="TESTER":
      self.jobsTable = "TESTJOBS"
      password="test"
    else:
      password = "".join(random.sample(string.letters,8))
      self.jobsTable="JOBS"
      station_id = 0
      
    submit_time=time.strftime("%Y-%m-%d %H:%M:%S")

    doc = xml.dom.minidom.parseString(xmlstring)
    solver = {}
    for tag in ['category','solver']:
      node = doc.getElementsByTagName(tag)
      if node and node[0]:
        solver[tag] = (NeosXML.getText(node[0]))
      else:
        self.logger.error("Error: '%s' tag not found in xml file" % tag)
        db.close()
        doc.unlink()
        return (0,"")
    node = doc.getElementsByTagName('inputMethod')
    if node and node[0]:
      solver['inputMethod'] = (NeosXML.getText(node[0]))
    else:
      node = doc.getElementsByTagName('inputType')
      if node and node[0]:
        solver['inputMethod'] = (NeosXML.getText(node[0]))
      else:
        self.logger.error("Error: 'inputMethod' tag not found in xml file")
        db.close()
        doc.unlink()
        return (0,"")
        
                               
    db.query('INSERT INTO %s (submit_time,user,clienthost,interface,password,solver_cat,solver_name,solver_input,status,station_id) VALUES("%s","%s","%s","%s","%s","%s","%s","%s","Waiting",%d)' % (self.jobsTable,submit_time,db.escape_string(user),db.escape_string(clienthost),interface,password,solver['category'],solver['solver'],solver['inputMethod'],station_id))

    jobNumber = db.query('SELECT LAST_INSERT_ID()')[0]
    db.setJobIn(jobNumber,xmlstring)
    
    node = doc.getElementsByTagName('priority')
    if interface == "TESTER":
      priority="test"
    elif node and node[0]:
      priority = (NeosXML.getText(node[0])).lower()
      if priority != "short":
        priority = "long"
    else:
      priority = "long"
      
    db.query('INSERT INTO QUEUE (job_number,status,priority,station_id) VALUES(%d,"Waiting","%s",%d)' % (jobNumber,priority,station_id))
    
    self.logger.debug("Starting email stuff")
    # register email forwarding
    if interface=='email' and user.find('@') > 0:
      db.query('INSERT INTO EMAIL (job_number, address) VALUES(%d,"%s")' % (jobNumber,db.escape_string(user)))

    else:
      node = doc.getElementsByTagName('email')
      if node and node[0]:
        address = NeosXML.getText(node[0])
        self.logger.debug("email = %s" %address)
        if address.find('@') > 0:
          db.query('INSERT INTO EMAIL (job_number, address) VALUES(%d,"%s")' % (jobNumber,db.escape_string(address)))
      else:
        self.logger.debug("no email tag")
        
          
      
    # send signal to scheduler
    result = db.query('SELECT pid FROM PROCS WHERE name="scheduler"')
    if result:
      try:
        os.kill(result[0],signal.SIGALRM)
      except OSError, (e):
        pass
    doc.unlink()
    return (jobNumber,password)
    

      
  def getResults(self,db,column,offset=0,blocking=True):
    # Get a lock if necessary (remember to release when done)
    if blocking:
      #self.logger.debug("acquiring %s lock" % column)
      NeosJob.locks.acquire(column,self.jobNumber)
    
    status = self.getJobStatus(db)
    if status == "Bad Password":
      if blocking:
        #self.logger.debug("releasing %s lock" % column)
        NeosJob.locks.release(column,self.jobNumber)
      return "%s is wrong password for job %d" % (self.password,self.jobNumber)

    if status == "Unknown Job":
      if blocking:
        #self.logger.debug("releasing %s lock" % column)
        NeosJob.locks.release(column,self.jobNumber)
      return "Job %d does not exist." % self.jobNumber

    if status == "Done":
      if column=='job_out':
        result = db.getJobOut(self.jobNumber)
      else:
        result = db.getJobResults(self.jobNumber)
      if blocking:
        #self.logger.debug("releasing %s lock" % column)
        NeosJob.locks.release(column,self.jobNumber)
      if result:
        return result[offset:]
      else:
        return "Job #%d is finished with no results" % self.jobNumber


    
    if blocking:
      #self.logger.debug("Blocked waiting for job #%d" % self.jobNumber)
      #self.logger.debug("waiting %s lock" % column)
      NeosJob.locks.wait(column,self.jobNumber)
      #self.logger.debug("UnBlocked job #%d" % self.jobNumber)
      #self.logger.debug("releasing %s lock" % column)
      NeosJob.locks.release(column,self.jobNumber)

    if column=='job_out':
      result = db.getJobOut(self.jobNumber)
    else:
      result = db.getJobResults(self.jobNumber)

    if result:
      return result[offset:]
    else:
      return ""
    
  
  def updateResults(self,db,column,outputString):
    if isinstance(outputString,xmlrpclib.Binary):
      outputString = outputString.data
    if column != "job_out" and column != "job_results":
      return "column must be 'job_results' or 'job_out'"

    # Make sure job is running
    status = self.getJobStatus(db)
    if status == "Bad Password":
      return "%s is wrong password for job %d" % (self.password,self.jobNumber)
    if status == "Done":
      return "Job %d has already finished." % self.jobNumber
    if status == "Unknown Job":
      return "Job %d does not exist." % self.jobNumber

    # get Lock
    #self.logger.debug("acquiring %s lock" % column)
    NeosJob.locks.acquire(column,self.jobNumber)
      
    if column=='job_results':
      self.removeFromQueue(db)
      if len(outputString):
        db.setJobResults(self.jobNumber,outputString)
      else:
        db.setJobResults(self.jobNumber,db.getJobOut(self.jobNumber))
    else:
      db.setJobOut(self.jobNumber,outputString,'a')

    self.logger.info("updateJobs %s %d" % (column,self.jobNumber))

    # Notify any listeners that output is available
    #self.logger.debug("notifying %s lock" % column)
    NeosJob.locks.notifyAll(column,self.jobNumber)

    # release Lock
    #self.logger.debug("releasing %s lock" % column)
    NeosJob.locks.release(column,self.jobNumber)


    # If job is done, also notify any job_out listeners
    if column == 'job_results':
      #self.logger.debug("acquiring jobout lock")
      NeosJob.locks.acquire('job_out',self.jobNumber)
      #self.logger.debug("notifying jobout lock")
      NeosJob.locks.notifyAll('job_out',self.jobNumber)
      #self.logger.debug("releasing jobout lock")
      NeosJob.locks.release('job_out',self.jobNumber)
      
    return "Results received: %d bytes" % len(outputString)


  def getJobStatus(self,db):
    result=db.query('SELECT status,password FROM %s WHERE number=%d' % (self.jobsTable,self.jobNumber))
    if result:
      status = result[0]
      realPassword = result[1]
      if self.password != realPassword and self.password != Variables.UBERPASSWORD:
        return "Bad Password"
      return status
    
    return "Unknown Job"


  def removeFromQueue(self,db):
    status = self.getJobStatus(db)
    if status=="Bad Password":
      return "%s is incorrect password for job %d" % (self.password,self.jobNumber)
    if status =="Done":
      return "Job %d has already finished." % (self.jobNumber)
    if status == "Unknown Job":
      return "Job %d does not exist." % (self.jobNumber)

    result = []
    try:
      # remove from queue
      db.query('DELETE FROM QUEUE WHERE job_number=%d' % (self.jobNumber))

      # Update JOBS entry 
      db.query('UPDATE %s SET end_time="%s",status="Done" WHERE number=%d' % (self.jobsTable,time.strftime("%Y-%m-%d %H:%M:%S"),self.jobNumber))

      # Set results to job_out if no results
      if db.getJobResults(self.jobNumber) =="":
        db.setJobResults(self.jobNumber,db.getJobOut(self.jobNumber))

      # get information for forwarding results if necessary
      result = db.query('SELECT user,interface from %s where number=%d'%(self.jobsTable,self.jobNumber))
      if result is None:
        raise Exception, ("No %s entry #%d" % (self.jobsTable,self.jobNumber))
    except Exception, (e):
      self.logger.error('Could not remove job #d from %s:\n%s' % (self.jobsTable,self.jobNumber,str(e)))
      return "Could not remove job #d from %s:\n%s" % (self.jobsTable,self.jobNumber,str(e))

    
  def printHtml(self,db):
    if self.password=="test":
      jobsTable="TESTJOBS"
      outdir="test"
    else:
      outdir="jobs"
      jobsTable="JOBS"
      
    self.logger.debug("Inside printHtml for job #%d" % self.jobNumber)
    if not os.access(os.path.join(Variables.NEOS_WWW_HOME,outdir),os.X_OK):
      os.mkdir(os.path.join(Variables.NEOS_WWW_HOME,outdir))

    # Get information on job from database
    result = db.query('SELECT solver_cat,solver_name,solver_input,start_time,end_time,station_id FROM %s WHERE number=%d' % (jobsTable,self.jobNumber))
    if result and len(result):
      
      solverString=":".join(result[0:3])
      start_time=result[3]
      end_time=result[4]
      station_id=result[5]
      self.logger.debug('solverString=%s' % solverString)

      if station_id==0:
        self.logger.debug('stationid==0')
        hostname="unscheduled"
      else:
        result = db.query('SELECT host_name FROM STATIONS WHERE ID=%d' % station_id)
        if result and len(result):
          hostname=result[0]
        else:
          self.logger.error("Error getting host_name from station %d" % station_id)
          return
    else:
      self.logger.error("Error getting job information for job# %d" % self.jobNumber)
      return


    #publish to web page
    self.logger.debug('publishing to web page')
    html = open(os.path.join(Variables.NEOS_WWW_HOME,outdir,
                             "%d.html" % self.jobNumber),"w")
    WebPrinter.printHtmlHeader(html,self.jobNumber,solverString,hostname,start_time,end_time)
    
    results = self.getResults(db,'job_results',blocking=False)

    if isinstance(results,xmlrpclib.Binary):
      html.write(results.data)
    else:
      html.write(results)
    html.write("""</PRE>
    <P><hr>
    <center><p>
    <a href ="%s" >
    <img src="%s/images/neos_home.jpg"
      border=0 alt="NEOS Server"></a>\n""" %
      (Variables.NEOS_WEB_SITE,Variables.NEOS_WEB_SITE))#")
    
    html.close()

        
                      
    
class NeosSolver:
  def __init__(self,log=None):
    if log:
      self.logger = log
    else:
      self.logger = EmptyLogger()

  def validateXml(self,xmlString):
    # first make sure string will parse
    try:
      self.logger.debug("Testing for xml parsability..")
      parser = xml.parsers.expat.ParserCreate()
      parser.Parse(xmlString)
      self.logger.debug("parsability OK")
    except xml.parsers.expat.ExpatError, (e):
      self.logger.debug("parsability failed")
      return "Error: xml string is not parseable.\n%s\n"%str(e)
      
    self.logger.debug("Testing for schema adherence:")
    #write xml string to temporary file
    xsdfile = os.path.join(Variables.INSTALL_DIR,'NEOS','neos.xsd')
    (fd,xmlfile) = tempfile.mkstemp()
    os.close(fd)
    f = open(xmlfile,"w")
    f.write(xmlString)
    f.close()

    #execute java validator (uses xerces)
    self.logger.debug("%s -cp %s/NEOS:%s XMLValidator %s %s" % (Variables.JAVA,Variables.INSTALL_DIR,Variables.XERCES_JARS,xmlfile,xsdfile))
    pipe = popen2.Popen3("%s -cp %s/NEOS:%s XMLValidator %s %s" % (Variables.JAVA,Variables.INSTALL_DIR,Variables.XERCES_JARS,xmlfile,xsdfile),1)
    
    input = pipe.tochild
    output = pipe.fromchild
    error = pipe.childerr

    input.close()
    outputClosed = 0
    errorClosed  = 0
    stop = 0
    msg = ""
    while not stop:
      try:
        ready = select.select([output, error], [], [])
        if len(ready[0]):
          if error in ready[0]:
            buf= error.readline()
            self.logger.debug(buf)
            if buf:
              msg += buf
            else:
              errorClosed = 1
          if output in ready[0]:
            buf = output.readline()
            self.logger.debug(buf)
            if buf:
              msg += buf
            else:
              outputClosed = 1
        if outputClosed and errorClosed:
          stop=1
      except IOError, (err):
        if err[0] == 4:
          self.logger.info("Child process dead, exiting")
        else:
          self.logger.exception(err)
        stop=1
      except Exception, (e):
        self.logger.exception(e)
        stop=1
    try:
      self.logger.debug("Closing output,error")
      output.close()
      error.close()
    except Exception, (e):
      self.logger.debug(e)
    
    status = pipe.wait()
    self.logger.debug("Exit status: %d" % status)
    if not status:
      msg = "OK"
    os.unlink(xmlfile)
    return msg

  def verifyPassword(self,db,category,solver,password):
    result = db.query('SELECT password FROM SOLVERS WHERE category="%s" AND name="%s" ' % (category,solver))
    if result and (password != result[0]):
      return False
    else:
      return True

  def exists(self,db,category,solver,input):
    result = db.query('SELECT COUNT(*) FROM SOLVERS WHERE category="%s" AND name="%s" AND input="%s"' % (category,solver,input))
    return (result[0] >= 1)
  
  def register(self,db,xmlString):
    message=""

    validationMsg = self.validateXml(xmlString)
    if validationMsg != "OK":
      return (False,"%s\nError validating XML file.\n  Please check file for errors, if you have questions send to %s" % (validationMsg,Variables.NEOS_COMMENTS))

    # Get solver information from xml
    parser = NeosXML.SolverParser(xmlString,self.logger)

    for inputMethod in parser.inputMethods:
      if not self.exists(db,parser.category,parser.solver,inputMethod):
        message += "Creating new solver %s:%s:%s\n" % \
                   (parser.category,parser.solver,inputMethod)
      else:
        # Validate password if solver already exists
        if self.verifyPassword(db,parser.category,parser.solver,
                               parser.password):
          message += "Updating existing solver %s:%s:%s\n" % \
                   (parser.category,parser.solver,inputMethod)
        else:        
          return (False,"Bad password for solver %s:%s:%s" % (parser.category,
                                                              parser.solver,
                                                              inputMethod))
      stations = ""
      for machine in parser.machines:
        stations += machine['user'] + "@" + machine['host_name'] + " "
      
      #remove old entries from STATIONS that will no longer be used
      results = db.queryAll('SELECT ID,user,host_name from STATIONS WHERE solver_cat="%s" AND solver_name="%s" AND solver_input="%s"' % (parser.category,parser.solver,inputMethod))
      for (id,user,host_name) in results:
        if stations.find(user+"@"+host_name) < 0:
          db.query('DELETE FROM STATIONS WHERE ID="%d"' % id)
          message += "%s:%s:%s was registered to run on %s@%s. Removing.\n" %\
                     (parser.category,parser.solver,inputMethod,user,host_name)
                      
                      

      # add/update entries to STATIONS
      for mach in parser.machines:
        # Check to see if any solvers are already running on the station
        result = db.query('SELECT port,pid from STATIONS WHERE user="%s" AND host_name="%s" AND pid >0 AND port >0' % (mach['user'],mach['host_name']))
        if result:
          port = result[0]
          pid = result[1]
        else:
          port=0
          pid=0

        result = db.query('SELECT ID from STATIONS WHERE solver_cat="%s" and solver_name="%s" and solver_input="%s" and user="%s" and host_name="%s"' % (parser.category,parser.solver,inputMethod,mach['user'],mach['host_name']))

        if result:
          # This station already exists, update
          station_ID = result[0]
          db.query('UPDATE STATIONS SET port=%d,pid=%d,time_limit=%d,mem_limit=%d,file_limit=%d,max_jobs=%d,filesize_limit=%d,priority="%s" WHERE user="%s" AND host_name="%s" AND solver_cat="%s" and solver_name="%s" and solver_input="%s"' % (port,pid,mach['time_limit'],mach['mem_limit'],mach['file_limit'],mach['max_jobs'],mach['filesize_limit'],mach['priority'],mach['user'],mach['host_name'],parser.category,parser.solver,inputMethod))
        else:
          # Add new station to database
          message+="Adding station %s@%s to solver %s:%s:%s\n" % \
                    (mach['user'],mach['host_name'],parser.category,
                     parser.solver,inputMethod)

          db.query('INSERT INTO STATIONS (solver_cat,solver_name,solver_input, user, host_name, port, pid,time_limit, mem_limit, file_limit, max_jobs, filesize_limit, priority) VALUES("%s","%s","%s","%s","%s",%d,%d,%d,%d,%d,%d,%d,"%s")' % (parser.category,parser.solver,inputMethod, mach['user'], mach['host_name'],port,pid,mach['time_limit'], mach['mem_limit'], mach['file_limit'], mach['max_jobs'],mach['filesize_limit'],mach['priority']))


      # Now find all stations running new solver
      stations = db.queryAll('SELECT ID,user,host_name,port FROM STATIONS WHERE solver_cat="%s" AND solver_name="%s" AND solver_input="%s"' % (parser.category,parser.solver,inputMethod))
      if stations and len(stations):
        station_list = "|".join([str(s[0]) for s in stations])
        self.logger.debug("station_list=%s" % station_list)
        
        for s in stations:
          message += "%s:%s:%s registered to run on %s@%s:\n" % (parser.category,parser.solver,inputMethod,s[1],s[2])
          port = int(s[3])
          host = xmlrpclib.ServerProxy("http://%s:%d" % (s[2],port))
          timeout = socket.getdefaulttimeout()
          socket.setdefaulttimeout(5)
          try:
            reply = host.ping()
          except Exception, e:
            reply = ""
          socket.setdefaulttimeout(timeout)
          if reply=="OK":
            message += "\tServer already running on host.\n"
          else:
            message += "\tServer needs to be started on this host.\n"

          # and add this list to SOLVERS table
          db.query('REPLACE INTO SOLVERS SET xmldescription="%s", contact="%s", password="%s", stations="%s", name="%s", category="%s", input="%s", status="Enabled"'%(db.escape_string(xmlString), parser.contact, parser.password, station_list, parser.solver, parser.category,inputMethod),logme=False)
            
      else:
        self.logger.error("Added new solver %s:%s:%s, but can't find stations running it" % (parser.category,parser.solver,inputMethod))
        message += "Error: No stations set for this solver"
      
    return (True,message)

  def registerExample(self,db,xmlstring,password):
    # Get solver information from example
    try:
      doc = xml.dom.minidom.parseString(xmlstring)
    except xml.parsers.expat.ExpatError, (e):
      return "Error parsing xml document:\n%s\n" % str(e) 
    solver = {}
    for tag in ['category','solver','inputMethod','exampleName']:
      node = doc.getElementsByTagName(tag)
      if node and node[0]:
        solver[tag] = (NeosXML.getText(node[0]))
      else:
        msg = "Error: '%s' tag not found in xml file\n" % tag
        self.logger.error(msg)
        doc.unlink()
        return msg

    node = doc.getElementsByTagName('exampleDescription')
    if node and node[0]:
      description = NeosXML.getText(node[0])
    else:
      description=""
      
    doc.unlink()
    category = solver['category']
    solverName = solver['solver']
    input = solver['inputMethod']
    exampleName = solver['exampleName']
    # check to make sure solver exists and password works
    if not self.exists(db,category,solverName,input):
      return "Solver %s:%s:%s does not exist\n" % (category,solverName,input)
    if not self.verifyPassword(db,category,solverName,password):
      return "Bad password for solver %s:%s" % (category,solverName)

    
    exampleFilename = db.setExample(exampleName,category,solverName,input,xmlstring)
    if exampleFilename is None:
      return "Error: NEOS could not create example file.\n"
    
    db.query('REPLACE INTO EXAMPLES SET name="%s", description="%s", solver_cat="%s", solver_name="%s", solver_input="%s", xml="%s"' % (exampleName, description, category, solverName, input, exampleFilename))

    WebPrinter.makeExamplePages(db,category,solverName,self.logger)

    return "example added, see web page %s/solvers/%s:%s/samples.html\n" % \
           (Variables.NEOS_WEB_SITE,category,solverName)

  def removeExample(self,db,category,solvername,input,password,examplename):
    if not self.exists(db,category,solvername,input):
      return "Solver %s:%s:%s does not exist\n" % (category,solvername,input)
    if not self.verifyPassword(db,category,solvername,password):
      return "Bad password for solver %s:%s" % (category,solvername)
    
    result = db.query('SELECT COUNT(*) from EXAMPLES WHERE name="%s" AND solver_cat="%s" AND solver_name="%s" AND solver_input="%s"' % (examplename,category,solvername,input))
    
    if result and len(result) and int(result[0]) == 1:
      db.query('DELETE from EXAMPLES WHERE name="%s" AND solver_cat="%s" AND solver_name="%s" AND solver_input="%s"' % (examplename,category,solvername,input))
      WebPrinter.makeExamplePages(db,category,solvername,self.logger)
      return "Removed example %s:%s:%s-%s from NEOS.\n" % (category,solvername,input,examplename)
    
    else:
      return "Unknown example %s:%s:%s-%s.\n" % (category,solvername,input,examplename)


             
  def disable(self,db,category,solver,input,password):
    # check if solver exists
    if not self.exists(db,category,solver,input):
      return "Solver %s:%s:%s does not exist\n" % (category,solver,input)
    if not self.verifyPassword(db,category,solver,password):
      return "Bad password for solver %s:%s" % (category,solver)
    
    result = db.query('SELECT status from SOLVERS where category="%s" AND name="%s" AND input="%s"' % (category,solver,input))
    if result:
      status = result[0]
    else:
      self.logger.debug("Bad state for solver %s:%s:%s" % (category,solver,input))
      return "Bad state for solver %s:%s:%s" % (category,solver,input)
    
    if status == "Disabled":
      return "Solver %s:%s:%s is already disabled." % (category,solver,input)
    else:
      db.query('UPDATE SOLVERS SET status="Disabled" WHERE category="%s" AND name="%s" AND input="%s"' % (category, solver, input))
      
      return "Solver %s:%s:%s is now disabled." % (category,solver,input)

  def enable(self,db,category,solver,input,password):
    # check if solver exists
    if not self.exists(db,category,solver,input):
      return "Solver %s:%s:%s does not exist\n" % (category,solver,input)
    if not self.verifyPassword(db,category,solver,password):
      return "Bad password for solver %s:%s" % (category,solver)
    
    result = db.query('SELECT status from SOLVERS where category="%s" AND name="%s" AND input="%s"' % (category,solver,input))
    if result:
      status = result[0]
    else:
      self.logger.debug("Bad state for solver %s:%s:%s" % (category,solver,input))
      return "Bad state for solver %s:%s:%s" % (category,solver,input)
    
    if status == "Enabled":
      return "Solver %s:%s:%s is already enabled." % (category,solver,input)
    else:
      db.query('UPDATE SOLVERS SET status="Enabled" WHERE category="%s" AND name="%s" AND input="%s"' % (category, solver, input))
      
      return "Solver %s:%s:%s is now enabled." % (category,solver,input)

  def getXML(self,db,category,solver,input):
    """
    returns the xml description of the given solver
    """
    result = db.query('SELECT xmldescription from SOLVERS LEFT JOIN CATEGORIES ON SOLVERS.category=CATEGORIES.abbreviation where (CATEGORIES.abbreviation="%s" OR CATEGORIES.full_name="%s") AND name="%s" AND input="%s"' % (category,category,solver,input))

    if result:
      # strip passwords from document before returning
      pattern = re.compile(r'<neos:password>.*</neos:password>')
      reply = pattern.sub('',result[0])
      return reply
    else:
      return "<document>Solver %s:%s:%s unknown.\n</document>" % \
             (category,solver,input)

  def getTemplate(self,db,category,solver,inputMethod):
    """
    returns the template for the given solver
    """
    result = db.query('SELECT xmldescription from SOLVERS LEFT JOIN CATEGORIES ON SOLVERS.category=CATEGORIES.abbreviation where (CATEGORIES.abbreviation="%s" OR CATEGORIES.full_name="%s") AND name="%s" AND input="%s"' % (category,category,solver,inputMethod))

    if result:
      solverxml = result[0]
      return WebPrinter.getTemplate(solverxml,inputMethod,self.logger)
    else:
      return "Solver %s:%s:%s unknown.\n" % \
             (category,solver,inputMethod)


  def removeSolver(self,db,category,solver,input,password):
    # check if solver exists
    if not self.exists(db,category,solver,input):
      return "Solver %s:%s:%s does not exist\n" % (category,solver,input)
    
    # Validate password if solver already exists
    if not self.verifyPassword(db,category,solver,password):
      return "Bad password for solver %s:%s:%s\n" % (category,solver,input)

    db.query('delete from SOLVERS where name="%s" and category="%s" and input="%s"' % (solver, category, input))

    db.query('delete from STATIONS where solver_name="%s" and solver_cat="%s" and solver_input="%s"' % (solver,category,input))

    # remove old directories
    shutil.rmtree(os.path.join(Variables.NEOS_WWW_HOME,"solvers",
                               "%s:%s:%s" % (category,solver,input)),True)

    
    return "Solver %s removed OK.\n"% (solver)


    
class NeosDB:
  conns=0
  def __init__(self,log=None,jobsdir=None):
    if jobsdir is None:
      self.jobsdir=Variables.JOBSDIR
    else:
      self.jobsdir=jobsdir
    if not os.access(self.jobsdir,os.X_OK):
      os.makedirs(self.jobsdir)
    if log:
      self.logger = log
    else:
      self.logger = EmptyLogger()
    
    try:
      self.db = MySQLdb.connect(host=Variables.MYSQL_HOST,
                                user=Variables.MYSQL_USER,
                                passwd=Variables.MYSQL_PASSWORD,
                                db=Variables.MYSQL_DATABASE)
      NeosDB.conns += 1
      self.logger.debug("Opening db connection. Number of connections is %d" % NeosDB.conns)
    except _mysql_exceptions.OperationalError, (errno, msg):
      self.logger.debug("Error connecting to database: %s" % msg)
      raise _mysql_exceptions.OperationalError, (errno, msg)

  def close(self):
    if self.db:
      self.db.close()
      NeosDB.conns -= 1
      self.logger.debug("Closing db connection. Number of connection is %d" % NeosDB.conns)
      self.db = None

  def queryAll(self,q,logme=True):
    if self.db is None:
      raise Exception("NeosDB error: cannot access closed db")
    cursor = self.db.cursor()
    if logme:
      self.logger.info(q)
    cursor.execute(q)
    return cursor.fetchall()

  def query(self,q,logme=True):
    if self.db is None:
      raise Exception("NeosDB error: cannot access closed db")
    cursor = self.db.cursor()
    if logme:
      self.logger.info(q)
    cursor.execute(q)
    return cursor.fetchone()
      
  def escape_string(self,s):
    return self.db.escape_string(s)

  def getJobFile(self,jobNumber,jobfile):
    jobfilename = os.path.join(self.jobsdir,str(jobNumber),jobfile)
    if os.access(jobfilename,os.R_OK):
      f = file(jobfilename,"r")
      buf=f.read()
      retval=""
      while buf:
        retval += buf
        buf = f.read()
      f.close()
      return retval
    else:
      return ""

  def getJobSize(self,jobNumber,jobfile):
    jobfilename = os.path.join(self.jobsdir,str(jobNumber),jobfile)
    filestat = os.stat(jobfilename)
    if filestat:
      size = filestat[stat.ST_SIZE]
    else:
      size=0

    return size

  def getJobOut(self,jobNumber):
    return self.getJobFile(jobNumber,'job.out')

  def getJobIn(self,jobNumber):
    return self.getJobFile(jobNumber,'job.in')

  def getJobResults(self,jobNumber):
    return self.getJobFile(jobNumber,'job.results')

  def getJobInSize(self,jobNumber):
    return self.getJobSize(jobNumber,'job.in')
  def getJobOutSize(self,jobNumber):
    return self.getJobOutSize(jobNumber,'job.out')
  def getJobResultsSize(self,jobNumber):
    return self.getJobResultsSize(jobNumber,'job.results')



  def setJobFile(self,jobNumber,data,jobfile,mode='w'):
    jobdir = os.path.join(self.jobsdir,str(jobNumber))
    if not os.access(jobdir,os.X_OK):
      try:
        os.makedirs(jobdir)
      except OSError, (e):
        self.logger.debug("Could not create job directory %s" % jobdir)
        raise OSError, (e)
                     
    try:
      jobfilename=os.path.join(self.jobsdir,str(jobNumber),jobfile)
      f = file(jobfilename,mode)
      f.write(data)
      f.close()
    except IOError, (e):
      self.logger.debug("Could not write to file %s." % jobfilename)
      raise IOError, (e)
    except OSError, (e):
      self.logger.debug("Could not write to file %s." % jobfilename)
      raise OSError, (e)

  def setJobResults(self,jobNumber,jobResults,mode='w'):
    self.setJobFile(jobNumber,jobResults,'job.results',mode)
    
  def setJobOut(self,jobNumber,jobOut,mode='w'):
    self.setJobFile(jobNumber,jobOut,'job.out',mode)
    
  def setJobIn(self,jobNumber,jobIn,mode='w'):
    self.setJobFile(jobNumber,jobIn,'job.in',mode)
  
  def setExample(self,exampleName,category,solverName,input,xmlstring):
    exdir = os.path.join(Variables.NEOS_WWW_HOME,"solvers",category+":"+solverName)
    if not os.access(exdir,os.X_OK):
      self.logger.debug("Could not access directory " + exdir)
      return None
    try:
      exfilename=os.path.join(exdir,exampleName+"_"+input+".txt")
      exfile = file(exfilename,'w')
      exfile.write(xmlstring)
      exfile.close()
      os.chmod(exfilename,0644)
      return exfilename
    except Exception, e:
      self.logger.exception(e)
      return None


        

  
