import xml.dom.minidom
import tempfile
import os
import base64
import zipfile
import gzip
import bz2




def getText(node):
  """
  Returns the text from the node of an xml document
  """
  s = ""
  if isinstance(node,(str,unicode)):
    return node
  if isinstance(node.nodeValue,(str,unicode)):
    return node.data
  elif node.hasChildNodes():
    for n in node.childNodes:
      s += getText(n)
  return s

def parseError(xmlstring):
  msg = "Error validating XML:\n"
  doc = xml.dom.minidom.parseString(xmlstring)
  for node in doc.getElementsByTagName("invalid"):
    line = node.getAttribute("line")
    char = node.getAttribute("char")
    text = getText(node).replace("{http://www.mcs.anl.gov/neos}","neos")
    msg += "At line %s, char %s: %s\n" % (line,char,text)

  return msg

def unzipIfNeeded(filename,logger=None):
  """
  Checks to see if the given filename is in zip, gzip, or bz2 compressed format.
  If it is, then it decompresses.
  """
  out = open("/home/testneos/playground/b64/zippedfile","wb")
  f = open(filename,'r')
  out.write(f.read())
  f.close()
  out.close()
  
  #zip files
  if zipfile.is_zipfile(filename):
    tempfilename = filename+"-tmp"
    tempfile = open(tempfilename,'wb')
    zipped = zipfile.ZipFile(filename,'r')
    namelist = zipped.namelist()
    tempfile.write(zipped.read(namelist[0]))
    tempfile.close()
    zipped.close()
    os.rename(tempfilename,filename)
    if logger is not None:
      logger.debug("Unzipped zip file")
    return
  else:
    logger.debug("Not a zip file.")

  #gz files
  try:
    gzipped = gzip.GzipFile(filename,'rb')

    # If not gzipped file, this throws an IOError
    gzipped.read(1)
    gzipped.seek(0)

    tempfilename = filename+"-tmp"
    tempfile = open(tempfilename,'wb')
    tempfile.write(gzipped.read())
    tempfile.close()
    gzipped.close()
    os.rename(tempfilename,filename)
    if logger is not None:
      logger.debug("Unzipped gzip file")
    return
  
  except IOError, e:
    if logger:
      logger.debug('Ignored: '+ str(e))
  except Exception, e:
    if logger:
      logger.debug(str(e))


  #bz2 files
  try:
    bz = bz2.BZ2File(filename,'rb')

    # If not bz2 file, this throws an IOError
    bz.read(1)
    bz.seek(0)

    tempfilename = filename+"-tmp"
    tempfile = open(tempfilename,'wb')
    tempfile.write(bz2.read())
    tempfile.close()
    bz2.close()
    os.rename(tempfilename,filename)
    if logger is not None:
      logger.debug("Unzipped bz2 file")

  except IOError, e:
    if logger:
      logger.debug('Ignored: '+str(e))
  except Exception, e:
    if logger:
      logger.debug(str(e))
    

  
    
    


def makeTarball(jobIn_xml, xmldescription,limits={},logger=None):
  """
  Convert the job input into the input files and then tarball them.
  Returns the text of the resultant tarball
  """
  jobIn_doc= xml.dom.minidom.parseString(jobIn_xml)
  solver_doc = xml.dom.minidom.parseString(xmldescription)
  items = []
    
  for node in solver_doc.getElementsByTagName("neos:input"):
    token = getText(node.getElementsByTagName("neos:token")[0])
    filename = getText(node.getElementsByTagName("neos:filename")[0])
    items.append([token,filename])

  tempdir=tempfile.mkdtemp()

  for item in items:
    for node in jobIn_doc.getElementsByTagName(item[0]):
      b64 = node.getElementsByTagName("base64")
      if len(b64):
        newfile = open(os.path.join(tempdir,item[1]),"wb")
        newfile.write(base64.decodestring(getText(b64[0])))
        newfile.close()
        # Check newfile for zip, gzip, bzip2 formats
        unzipIfNeeded(os.path.join(tempdir,item[1]),logger)
      else:               
        newfile = open(os.path.join(tempdir,item[1]),"w")
        newfile.write(getText(node))
        newfile.close()
      
  for limit in limits.keys():
    newfile = open(os.path.join(tempdir,"neos-"+limit),"w")
    newfile.write(str(limits[limit])+"\n")
    newfile.close()
      
  # Now combine the files into a tarball
  filelist = [x[1] for x in items]
  filelist.extend(["neos-"+i for i in limits.keys()])
  try:
    import tarfile
    tar = tarfile.open(os.path.join(tempdir,"jobin.tar"), "w")
    for filename in filelist:
      if os.access(os.path.join(tempdir,filename), os.R_OK):
        tar.add(os.path.join(tempdir,filename), filename)
    tar.close()
  except ImportError, (errno, msg):
    os.system("tar -cf %s %s" % (os.path.join(tempdir,"jobin.tar"),
                                " ".join(filelist)))

  # Read the tarball as regular file
  tar = open(os.path.join(tempdir,"jobin.tar"),"rb")
  tarball = tar.read()
  tar.close()

  try:
    for filename in filelist:
      if os.access(os.path.join(tempdir,filename), os.R_OK):
        os.remove(os.path.join(tempdir,filename))

    os.remove(os.path.join(tempdir,"jobin.tar"))
    os.removedirs(tempdir)
  except:
    pass

  jobIn_doc.unlink()
  solver_doc.unlink()
  return tarball




class SolverParser:
  def __init__(self,xmlstring,logger=None):
    self.xml = xmlstring
    self.logger = logger
    self.doc = xml.dom.minidom.parseString(xmlstring)
    self.examples=[]
    self.items=[]
    self.machines = []
    self.abstract=""

    node = self.doc.getElementsByTagName("neos:abstract")
    if node and len(node):
      self.abstract = getText(node[0])
    # The password might be removed from xml file
    node = self.doc.getElementsByTagName("neos:password")
    if node and len(node):
      self.password = getText(node[0])
    node = self.doc.getElementsByTagName("neos:link")
    if node and len(node):
      self.link = getText(node[0])
    node = self.doc.getElementsByTagName("neos:contact")
    if node and len(node):
      self.contact = getText(node[0])
    node = self.doc.getElementsByTagName("neos:category")
    if node and len(node):
      self.category = getText(node[0])
    else:
      raise Exception("Error, no category tag in xml file")
    
    node = self.doc.getElementsByTagName("neos:solver")
    if node and len(node):
      self.solver = getText(node[0])
    else:
      raise Exception("Error, no solver tag in xml file")
    self.token = self.category + ":" + self.solver
    self.inputMethods = []
    for element in self.doc.getElementsByTagName("neos:inputMethod"):
      self.inputMethods.append(getText(element))

    self.examples = self.doc.getElementsByTagName("neos:example")
    self.extractInputItems()
    self.extractMachines()


  def generateExamples(self):
    example_list=[]
    for ex_node in self.examples:
      outstring = ""
      name = ex_node.getAttribute("name")
      inputType = ex_node.getAttribute("inputType")
      
      outstring+="<document>\n<category>%s</category>\n"% (self.category)
      outstring += "<solver>%s</solver>\n" % (self.solver)
      outstring += "<inputType>%s</inputType>\n" % (inputType)
      outstring += "<priority>short</priority>\n" 
      for item in self.items:
        valueList = ex_node.getElementsByTagName(item.token)
        if len(valueList) > 0:
          b64 = valueList[0].getElementsByTagName("base64")
          if len(b64):
            outstring+="<%s><base64>%s</base64></%s>\n"%(item.token,value,item.token)
          else:
            value = getText(valueList[0])
            if (value.find("<")>=0) or (value.find(">")>=0) or \
                 (value.find("&")>=0):
              outstring+="<%s><![CDATA[%s]]></%s>\n"%(item.token,value,item.token)
            else:
              outstring+="<%s>%s</%s>\n"%(item.token,value,item.token)
      outstring +="</document>\n"
      example_list.append((name,outstring))
    return example_list


    
  def extractInputItems(self):
    for node in self.doc.getElementsByTagName("neos:input"):
      
      type = getText(node.getAttribute("TYPE"))
      newItem = Item(type)

      n = node.getElementsByTagName("neos:token")
      if n and len(n):
        newItem.setToken(getText(n[0]))
      else:
        raise Exception("Error: no token tag in xml file for input item")

      n = node.getElementsByTagName("neos:filename")
      if n and len(n):
        newItem.setFilename(getText(n[0]))
      else:
        raise Exception("Error: no filename tag in xml file for input item")

      n = node.getElementsByTagName("neos:text")
      if n and len(n):
        newItem.setPrompt(getText(n[0]))
      else:
        n = node.getElementsByTagName("neos:prompt")
        if n and len(n):
          newItem.setPrompt(getText(n[0]))

      n = node.getElementsByTagName("neos:default")
      if n and len(n):
        newItem.setDefault(getText(n[0]))
      helpElements = node.getElementsByTagName("neos:help")
      if helpElements and len(helpElements):
        newItem.setHelp(getText(helpElements[0]))
      for option in node.getElementsByTagName("neos:option"):
        newItem.addOption(option)
      self.items.append(newItem)
                           

  def extractMachines(self):
    self.quicksolve=0
    for machineNode in self.doc.getElementsByTagName("neos:machine"):
      machine = {'max_jobs':2, 'time_limit':28800,
                 'mem_limit':10000000l, 'file_limit':100,
                 'filesize_limit':10000000l,'priority':'long'}

      node = machineNode.getElementsByTagName("neos:hostname")
      machine['host_name'] = getText(node[0])
      
      node = machineNode.getElementsByTagName("neos:user")
      machine['user'] = getText(node[0])
      
      node = machineNode.getElementsByTagName("neos:max_jobs")
      if node:
        machine['max_jobs'] = int(getText(node[0]))

      node = machineNode.getElementsByTagName("neos:time_limit")
      if node:
        machine['time_limit'] = int(getText(node[0]))

      node = machineNode.getElementsByTagName("neos:mem_limit")
      if node:
        machine['mem_limit'] = long(getText(node[0]))

      node = machineNode.getElementsByTagName("neos:file_limit")
      if node:
        machine['file_limit'] = int(getText(node[0]))

      node = machineNode.getElementsByTagName("neos:filesize_limit")
      if node:
        machine['filesize_limit'] = long(getText(node[0]))

      node = machineNode.getElementsByTagName("neos:priority")
      if node and getText(node[0]).lower()=="short":
        machine['priority'] = "short"

      self.machines.append(machine)


    
      
      

class Item:
  def __init__(self,type):
    self.options = []
    self.filename=""
    self.prompt=""
    self.default=""
    self.token=""
    self.help=""
    self.url = None
    self.whatami=type

  
  def setToken(self,s):
    self.token=s
  def setFilename(self,s):
    self.filename=s
  def setHelp(self,s):
    self.help=s
  def setPrompt(self,s):
    self.prompt=s
  def setDefault(self,s):
    self.default=s
  def setUrl(self,s):
    self.url = s
  def addOption(self,option):
    checked = option.getAttribute("default")
    value = option.getAttribute("value")
    self.options.append((getText(option),checked,value))

  def getWhatami(self):
    return self.whatami
  def getToken(self):
    return self.token
  def getFilename(self):
    return self.filename
  def getHelp(self):
    return self.help
  def getPrompt(self):
    return self.prompt
  def getDefault(self):
    return self.default
  def getUrl(self):
    return self.url
  def getOptions(self):
    return self.options
  def getTextHelp(self):
    """
    If help string is html formatted, then return just the plain text
    """
    from HTMLParser import HTMLParser

    class MyHTMLParser(HTMLParser):
      def __init__(self):
        HTMLParser.__init__(self)
        self.msg = ""
      def handle_start_tag(self, tag,attrs):
        msg += self.get_starttag_text()

    try:
      parser = MyHTMLParser()
      parser.feed(self.help)
      parser.close()
    except Exception, e:
      pass
    if parser and parser.msg:
      return parser.msg
    else:
      return self.help
  

