#!/usr/bin/env python
import os
import NeosXML
import NeosTools
from config import Variables
import logging

def printHtmlHeader(page, jobNumber, solver,station,starttime,endtime=""):
  if jobNumber is not None:
    page.write("<title>NEOS Job #%d</title>\n" % jobNumber)
  else:
    page.write("<title>NEOS</title>\n")
               
  page.write("""
<BODY BGCOLOR='%s'>
<center>
<a href ="%s" >
<img src="%s/images/globe_small.jpg"
alt="NEOS Server Home" border=0></a></center>
<P><hr><br>
<PRE>
*************************************************************

   NEOS Server Version 4.0
"""%(Variables.NEOS_WEB_BGCOLOR,Variables.NEOS_WEB_SITE,Variables.NEOS_WEB_SITE))
  if jobNumber is not None:
    page.write("""
   Job#   : %d
   Solver : %s
   Start  : %s
   End    : %s
   Host   : %s
""" %  (jobNumber,solver,starttime,endtime,station))

  page.write(Variables.NEOS_ANNOUNCEMENTS)
  page.write("""
   Disclaimer:

   This information is provided without any express or
   implied warranty. In particular, there is no warranty
   of any kind concerning the fitness of this
   information  for any particular purpose.

*************************************************************\n""")

class WebPrinter:
  def __init__(self,xmlstring=None,logger=None):
    if logger is None:
      self.logger = logging.getLogger('webprinter')
      hdlr = logging.FileHandler("webprinter.log")
      formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
      hdlr.setFormatter(formatter)
      self.logger.addHandler(hdlr)
      self.logger.setLevel(logging.DEBUG)
    else:
      self.logger=logger

      
    if xmlstring is not None:
      self.solverdoc = NeosXML.SolverParser(xmlstring,self.logger)
    else:
      self.solverdoc = None

    self.webdir = os.path.join(Variables.NEOS_WWW_HOME,"solvers")
    logger.debug("Making webpages in " + self.webdir)
    if not os.access(self.webdir,os.X_OK):
      os.makedirs(self.webdir,mode=0755)
    if not os.access(self.webdir,os.X_OK):
      logger.error("Did not create directory %s",self.webdir)

    
    
  def printExamplePages(self):
    solverdir = os.path.join(self.webdir,self.solverdoc.token)

    self.logger.debug("printing example pages.")
    
    db = NeosTools.NeosDB(self.logger)
    results = db.queryAll('SELECT name,solver_input from EXAMPLES WHERE solver_cat="%s" AND solver_name="%s" ORDER BY solver_input' % (self.solverdoc.category,self.solverdoc.solver))
    db.close()
    
    if results is None or len(results)==0:
      results = []
    
      
    # print index page
    exampleindex = open(os.path.join(solverdir,"samples.html"),"w")
    self.logger.debug("Printing example index.")
    exampleindex.write('''\
    <HTML>\n\
    <HEAD>\n\
    <TITLE> %s Sample Submissions </TITLE>\n\
    <META name="description" \n\
    content="%s NEOS Server: Sample Submissions Form">\n\
    <META name="keywords" \n\
       content="%s, NEOS Server, optimization, \n\
       algorithms, software, problem solving environments">\n\
    </HEAD>\n\
    <BODY BGCOLOR="%s" >\n\
    <A HREF="%s"><IMG align=left border=0 SRC="%s/images/globe_small.jpg"\n\
       alt="NEOS Server Home"></A>\n\
    <P><P><CENTER>\n\
    <H1>Sample Submissions<p>%s</H1>\n\
    </CENTER>\n\
    <br clear=left>\n\n\
    <br><hr><br>\n\
    <FORM METHOD="POST"\n\
    ACTION="%s/nph-neos-solver.cgi">\n\
    Select one of the following problems to submit to %s.  The job results will be displayed on your browser.  The tokens to delimit files as shown in the sample submissions are inserted automatically when using the Web forms.<ul>\n\
    <TABLE  border=0>\n\
    ''' % (self.solverdoc.solver,self.solverdoc.solver, self.solverdoc.solver,
           Variables.NEOS_WEB_BGCOLOR,
           Variables.NEOS_WEB_SITE,Variables.NEOS_WEB_SITE,self.solverdoc.solver,
           Variables.NEOS_CGI_SITE,self.solverdoc.solver))

    CHECKED = "CHECKED"

    if not results or len(results)==0:
      exampleindex.write('<p><font color=red>No examples have been provided.  Please contact %s for help.</font></p>' % self.solverdoc.contact)
      
    for (name,input) in results:
      exampleindex.write('''\
      <TR><TD><INPUT TYPE=radio NAME="example" VALUE="%s_%s" %s>\n\
      <A href="%s_%s.txt">%s</A> <small>[%s]</small></TD></TR>\n\
      ''' % (name,input,CHECKED,name,input,name,input))
      CHECKED=""


    exampleindex.write('''</TABLE>\n\
    </ul>\n\
    <hr><P>\n\
    Enter your e-mail address if you would like a copy of the \n\
    sample submission and job results mailed to you.<P><center>\n\
    <input type="hidden" name="category" value="%s">\n\
    <input type="hidden" name="solver" value="%s">\n\
    <table cols=2><tr><td colspan=2 align=center>e-mail address:<BR> <INPUT TEXT NAME="email" size=36"><P></td>\n\
    </tr><tr><td align=right><INPUT TYPE="submit" VALUE="Submit to NEOS"></td><td align=left><INPUT TYPE="reset" VALUE="Clear this Form">\n\
    </td></tr></table>\n\
    </center>\n\
    <hr>\n\n\
    <center><a href ="%s" ><img src="%s/images/neos_home.jpg" border=0 alt="NEOS Server Home" hspace=40></a>\n\
    <br>\n\
    <a href="./solvers/%s/server-comment.html">\n\
    <img border=0 src="%s/images/comments.jpg"></a>\n\
    </center><p>\n\
    </BODY>\n\
    </HTML>\n''' % (self.solverdoc.category,self.solverdoc.solver,
                    Variables.NEOS_WEB_SITE,Variables.NEOS_WEB_SITE,
                    self.solverdoc.solver,Variables.NEOS_WEB_SITE))
    exampleindex.close()
    os.chmod(os.path.join(solverdir,"samples.html"),0644)




  def printPage(self):
    for inputType in self.solverdoc.inputMethods:
      self.logger.debug("Printing page for %s:%s" % (self.solverdoc.token,inputType))
      directory= os.path.join(self.webdir,self.solverdoc.token)
      try:
        if not os.access(directory,os.X_OK):
          self.logger.debug("Making directory " + directory)
          os.makedirs(directory,mode=0755)
        self.logger.debug("Making file "+os.path.join(directory,inputType+".html"))
        webpage = open(os.path.join(directory,inputType+".html"),"w")
        webpage.write(self.printHeader(inputType))
        webpage.write(self.solverdoc.abstract)
        index = 1
        for i in self.solverdoc.items:
          if i.whatami == "radio":
            webpage.write(self.printRadio(i,index))
          elif i.whatami == "textfield":
            webpage.write(self.printTextfield(i,index))
          elif i.whatami == "textarea":
            webpage.write(self.printTextarea(i,index))
          elif i.whatami == "file":
            webpage.write(self.printFile(i,index))
          elif i.whatami == "checkbox":
            webpage.write(self.printCheckbox(i,index))
          index += 1

        webpage.write(self.printFooter(inputType))
        webpage.close()

        os.chmod(os.path.join(directory,inputType+".html"),0744)
        
      except Exception, (error):
        self.logger.exception(error)
        raise Exception, error

  
  def printRadio(self,item,index):
    retval = item.getHelp() + '<br>\n<b>' + item.getPrompt() + ':</b><br>'
    checked = False
    retval += '<ul>\n'
    for option in item.getOptions():
      retval += '<input TYPE="RADIO" NAME="field.'+str(index)+'" '
      retval += 'VALUE="'+option[2] + '" '
      if option[1] and not checked:
        retval += ' CHECKED'
        checked = True
      retval += '>' + option[0] + '<br>\n'
    retval += '</ul>\n'
    retval += '<br><hr>\n'
    return retval

  def printCheckbox(self,item,index):
    retval = item.getHelp() + '<br>\n<b>' + item.getPrompt() + ':</b><br>'
    checked = False
    retval += '<input TYPE="checkbox" NAME="field.%d" VALUE="%s">' % (index,item.getDefault())
    retval += '<br><hr>\n'
    return retval



  def printTextarea(self,item,index):
    retval = item.getHelp() + '<br>\n<b>' + item.getPrompt() + ':</b><br>'
    retval += '<textarea NAME="field.' + str(index) + '" '
    retval += 'ROWS=8 COLS=60>' + item.getDefault()
    retval += '\n</textarea>\n<br><hr>\n'
    return retval

  def printTextfield(self,item,index):
    retval = item.getHelp() + '<br>\n<b>' + item.getPrompt() + ':</b><br>'
    retval += '<input TEXT NAME="field.' + str(index) + '" '
    retval += 'size=50 VALUE="%s">\n' % item.getDefault()
    retval += '<br><hr>\n'
    return retval
    
  def printFile(self,item,index):
    retval = item.getHelp() + '<br>\n<b>' + item.getPrompt() + ':</b><br>'
    retval += '<input TYPE="file" NAME="field.' + str(index) + '" '
    retval += 'SIZE=50>\n'
    retval += '<br><hr>\n'
    return retval

  def printFooter(self,inputType):
    self.logger.debug("printing footer")
    retval=""
    
    # check if priority queue available for this solver
    for machine in self.solverdoc.machines:
      if machine.has_key('priority') and machine['priority'] == "short":
        retval+='''\
        <input TYPE="checkbox" NAME="priority" VALUE="short">\
        Put in priority queue<br>\n\
        '''
        break
    

      
    retval += '''\
    <center><table cols=2><tr><td colspan=2 align=center>e-mail address:<br>\n\
    <input type="hidden" name="category" value="%s">\n\
    <input type="hidden" name="solver" value="%s">\n\
    <input type="hidden" name="inputMethod" value="%s">\n\
    <input TEXT NAME="email" SIZE=36"><p></td></tr>\n\
    <tr><td ALIGN=right><input TYPE="submit" VALUE="Submit to NEOS"></td>\n\
    <td ALIGN=left><input TYPE="reset" VALUE="Clear this Form"></td></tr>\n\
    </table></center><p><hr>\n\
    <center><a href="%s" ><img SRC="%s/images/neos_home.jpg" border=0 \n\
    alt="NEOS Server Home" HSPACE=40></a><br>\n\
    <a href="%s/server-comment.html">\n\
    <img border=0 src="%s/images/comments.jpg" \n\
    ALT="Submit comments and questions"></a>\n\
    </center>\n
    <p><br>%s\n</body></html>\n''' % \
    (self.solverdoc.category,
     self.solverdoc.solver,
     inputType,
     Variables.NEOS_WEB_SITE,
     Variables.NEOS_WEB_SITE,
     Variables.NEOS_WEB_SITE,
     Variables.NEOS_WEB_SITE,
     Variables.NEOS_WEB_DISCLAIMER)
    return retval

    
  def printHeader(self,inputType):
    retval =  '''\
    <html>\n\
    <head>\n\
    <title> NEOS Server: %s</title>\n\
    <meta name="description"\n\
    content="NEOS Server: %s">\n\
    <meta name="keywords"\n\ 
    content="%s NEOS Server, optimization,\n\
    algorithms, software, problem solving environments"></head>\n\
    <body BGCOLOR="%s" >\n\
    <FORM METHOD="POST"\n\
    ACTION="%s/nph-neos-solver.cgi" \n\
    ENCTYPE="multipart/form-data" NAME="WWW Interface">\n\
    <table border=0 width=100%% cellspacing=0 cellpadding=0>\n\
    <tr><td></td><td width=100%%></td><td></td>\n\
    <tr valign=top><td align=left>\n\
    <a href="%s"><img border=0 SRC="%s/images/globe_small.jpg" alt="NEOS Server Home" valign=top></a>\n\
    </td><td align=right colspan=2>\n\
    <table cellpadding=5 cellspacing=1 border=2>\n\
    <tr align=left valign=top>\n\
    \n\
    <td>\n\
    <h3>NEOS Interfaces to %s</h3>\n\
    \n\
    <a href="">\n\
    <b>WWW Form</b></a>\n\
    &amp; <a href="samples.html"><b>Sample Submissions</b></a>\n\
    <br>\n\
    <a href="%s-help.html">\n\
    <b>E-mail</b></a>\n\
    <br>\n\
    <a href="%s-help.html">\n<b>XML-RPC</b></a>\n\
    </td></tr></table>\n\
    </td>\n\
    <tr valign=bottom>\n\
    <td align=left colspan=2>\n\
    <br><p>\n''' % (self.solverdoc.solver,
                    self.solverdoc.solver,
                    self.solverdoc.solver,
                    Variables.NEOS_WEB_BGCOLOR,
                    Variables.NEOS_CGI_SITE,
                    Variables.NEOS_WEB_SITE,
                    Variables.NEOS_WEB_SITE,
                    self.solverdoc.solver,
                    inputType,inputType )
                    
    
    if hasattr(self.solverdoc,"link"):
      retval += '''<a href="%s"><h1>%s</h1></a>\n''' % (self.solverdoc.link,self.solverdoc.solver)
    else:
      retval += '''<h1>%s</h1>\n''' % (self.solverdoc.solver)

    retval += '''\
    </td><td align=right>\n\
    </td></tr></table>\n\
    \n\
    <br clear=all><p><br>\n\
    \n\
    \n'''

    return retval
    
  def getHelpString(self,plaintext=False):
    msg = ""
    for item in self.solverdoc.items:
      if plaintext:
        msg += "%s\n---------------\n%s\n" % (item.getToken(),item.getTextHelp())
      else:
        msg += "<b>%s</b><br>\n%s<br>\n" % (item.getToken(),item.getHelp())
    return msg + "\n\n"
  
  def getHelpTemplate(self,inputType,plaintext=False):
    msg =  """<document>
<category>%s</category>
<solver>%s</solver>
<inputMethod>%s</inputMethod>\n
""" % (self.solverdoc.category,
       self.solverdoc.solver,
       inputType)
    for item in self.solverdoc.items:
      msg += "<%s><![CDATA[\n...Insert Value Here...\n]]></%s>\n\n" %\
             (item.getToken(),item.getToken())
      
    msg += "</document>\n"
    if plaintext:
      return msg
    else:
      return "<pre><code>%s</code></pre>" % msg.replace('<','&lt;').replace('>','&gt;')
  
  def printHelpPage(self):
    for inputType in self.solverdoc.inputMethods:
      self.logger.debug("Printing help page for %s:%s" % (self.solverdoc.token,inputType))
      directory= os.path.join(self.webdir,self.solverdoc.token)
      try:
        if not os.access(directory,os.X_OK):
          self.logger.debug("Making directory " + directory)
          os.makedirs(directory,mode=0755)
        self.logger.debug("Making file "+os.path.join(directory,inputType+"-help.html"))
        webpage = open(os.path.join(directory,inputType+"-help.html"),"w")
        webpage.write("""
<html>

<head><title>Email and XML-RPC Interface to %s:%s</title>
<BASE HREF =" %s">
</head>

<BODY BGCOLOR="%s"> 

<A HREF=""><IMG align=left SRC="%s/images/globe_small.jpg" ALT="NEOS Server Home" border=0></A>

<br clear=all>

<br><hr><br>

Follow these steps to submit a job to NEOS via email or XML-RPC clients:
<OL>
<li> Save the following template using your browser's copy and paste tools
<li> Using your favorite editor, plug your
  data files and parameters into the message in the indicated spots
<li> Descriptions of the input are written with the web interface in mind.
If the description asks for the name of a file, then you will instead need
to insert the contents of the file itself.
<li> The <code>&lt;![CDATA[ ... ]]&gt;</code> tag is an escape indicator.  This
will ensure that any special characters in your inputs (such as &lt;, &gt;,
&amp;, etc.) will be interpreted correctly.

  
</OL>

<p>For email, send this file as the message body or as a plain text attachment to <t> %s</t>.  Do not send .doc, .rtf, .xsw, or other files that aren't plain text.  The job results will be mailed to you.


<p>For XML-RPC, you need to submit the contents of this file to NEOS with the
<code>submitJob()</code> method, and retrieve the results with the <code>getResults()</code> method.  Go <a href="Web/NEOS-API.html">here</a> for more information on using the XML-RPC interface to NEOS.
<p>
"""    % (self.solverdoc.token,inputType,Variables.NEOS_WEB_SITE,
          Variables.NEOS_WEB_BGCOLOR,Variables.NEOS_WEB_SITE,Variables.NEOS_MAIL)) #"
        webpage.write(self.getHelpTemplate(inputType))
        webpage.write("<hr><h4>Description of input values</h4>\n")
        webpage.write(self.getHelpString())
        webpage.write(Variables.NEOS_WEB_DISCLAIMER)
        webpage.write("</body></html>")
      except Exception, e:
        self.logger.exception(str(e))
        raise Exception, e



def makeWebPages(xmlstring,logger):
  wp = WebPrinter(xmlstring,logger)
  wp.printPage()
  wp.printHelpPage()
  wp.printExamplePages()

def makeExamplePages(db,category,solver,logger):
  result = db.query('SELECT xmldescription from SOLVERS WHERE category="%s" AND name="%s"' % (category,solver))
  if not result or not len(result):
    logger.debug("Could not get any xmldescription for %s:%s" % (category,solver))
    return
  xmlstring = result[0]
  
  wp = WebPrinter(xmlstring,logger)
  wp.printExamplePages()

def getTemplate(xmlstring,input,logger):
  wp = WebPrinter(xmlstring,logger)
  return wp.getHelpTemplate(input,plaintext=True)

def makeTestCatalog(db,logger):
  makeCatalog(db,logger,test=True)
  
def makeCatalog(db,logger,test=False):
  if logger is None:
    logger = NeosTools.EmptyLogger()

  # make catalog page
  logger.debug("Editing NEOS solver catalog")
  if not os.access(os.path.join(Variables.NEOS_WWW_HOME,"solvers"),os.X_OK):
    os.mkdir(os.path.join(Variables.NEOS_WWW_HOME,"solvers"))
                   
  if test:
    catalog = open(os.path.join(Variables.NEOS_WWW_HOME,"solvers",
                                "test.html"),"w")
  else:
    catalog = open(os.path.join(Variables.NEOS_WWW_HOME,"solvers",
                                "index.html"),"w")
                              
  # Write header of catalog web page
  catalog.write("""
  <html>

  <head>

  <title> NEOS Solvers</title>

  <style type="text/css">
  <!--
  UL.blank {list-style-type: none}
  -->
  </style>

  <meta name="description" 
  content="The NEOS Server optimization solvers represent the
                state-of-the-art in computational optimization.
                Optimization problems are solved automatically with
                minimal input from the user. Users only need a definition
                of the optimization problem; all additional information
                required by the optimization solver is determined
                automatically.">

  <meta name="keywords" content="optimization, algorithms,
                software, problem solving environments, linear programming,
                nonlinear programming, constrained optimization, 
                linear networks, stochastic linear programming, 
                complementarity, nonlinear equations, systems of equations, 
                least squares, data fitting, automatic differentiation">


  </head>

  <body BGCOLOR="%s">
  <center>
  <a HREF="../index.html">
  <img border=0 
  SRC="%s/images/neos-world.gif" ALT="NEOS Server Home"></a>

  </center>
  <p>
  Choose a solver for the type of optimization problem that you want to
  solve.  If you are not sure of the type of optimization problem,
  consult the
  <a href="http://www.mcs.anl.gov/otc/Guide/OptWeb">
  Optimization Tree </a>
  of the NEOS Guide
  for information on optimization problems.
  The choice of solver is then dictated by the language used to define the
  optimization problem.  



  <p>
  Each solver has sample problems and background information on the solver.
  Be sure to submit a sample problem to get a feel for how to submit
  optimization problems to NEOS.
  If you encounter problems, consult the 
  <a href="../faq.html">
  NEOS Server FAQ,
  </a>
  or send mail to us by clicking on the 
  <b> Comments and Questions </b> icon at the bottom of the page.

  <br><hr><p>
  <ul>\n
    <li><h4>Administration</h4>
    <ul>
      <li><a href="../admin.html">view job queue</a>
      <li><a href="../admin.html">view job status/results</a>
      <li><a href="../admin.html">kill/dequeue job</a>
    </ul>
  """ % (Variables.NEOS_WEB_BGCOLOR,Variables.NEOS_WEB_SITE))

  # write entries for each solver
  if test:
    category_list = db.queryAll('select abbreviation,full_name from CATEGORIES where full_name="test"')
  else:
    category_list = db.queryAll('select abbreviation,full_name from CATEGORIES where full_name != "kestrel" AND full_name != "test" ORDER BY abbreviation')
    
  for category in category_list:
    abbreviation = category[0]
    full_name = category[1]
    results = db.queryAll('select name,input from SOLVERS where category="%s" AND status="Enabled" ORDER BY name,input'%abbreviation)
    catalog.write('<li><h4>%s</h4>\n<ul class="BLANK">\n'%full_name)
    oldname=""
    for result in results:
      name = result[0]
      input = result[1]
      if name != oldname:
        catalog.write('<li>%s [<a href="./%s:%s/%s.html">%s Input</a>]' %  (name,abbreviation,name,input,input))
        oldname=name
      else:
        catalog.write('[<a href="./%s:%s/%s.html">%s Input</a>]' %
                      (abbreviation, name,input,input))

    catalog.write('</ul>\n</li>\n')
  catalog.write('</ul>\n')

  # Write footer of catalog web page
  catalog.write('''
  <center>
  <a href="../index.html"><img src="%s/images/neos_home.jpg" border=0
  alt="NEOS Server Home" hspace=40></a>
  <br>
  <a href="../server-comment.html">
  <img border=0 src="%s/images/comments.jpg"
  ALT="Submit comments and questions"></a>
  </center><p>
  <p><br>
  %s
  </body>
  </html>
  ''' % (Variables.NEOS_WEB_SITE,Variables.NEOS_WEB_SITE,
         Variables.NEOS_WEB_DISCLAIMER))

  
  
