#!/usr/bin/env python
import time
import MySQLdb
import _mysql
import socket
import signal
import logging
import logging.handlers
import os
import sys
import xmlrpclib

import NeosTools

from config import Variables

global logger

class Checker:
  def __init__(self):
    self.db = NeosTools.NeosDB(logger)

  def __del__(self):
    self.db.close()

  def checkQueue(self):
    # Check jobs in queue
    results = self.db.queryAll('SELECT job_number, station_id, pid FROM QUEUE WHERE status="running"')

    for result in results:
      jobNumber = int(result[0])
      if result[1] is not None:
        station_id = int(result[1])
      else:
        station_id=0

      if result[2] is not None:
        pid = int(result[2])
      else:
        pid=0
        
      logger.debug("job=%d id=%d pid=%d" % (jobNumber, station_id, pid))
      try:
        alive = NeosTools.isJobAlive(jobNumber, station_id, pid, logger)
      except Exception, (err):
        logger.exception(err)
        alive=False

      if not alive:
        logger.info("job %d does not exist on %s, removing from queue"
                    % (jobNumber, station_id))
        self.db.query('DELETE FROM QUEUE WHERE job_number = %d' % jobNumber)

  def checkProcess(self,process):
    logger.debug('Checking for process %s' % process)
    results = self.db.query('SELECT pid from PROCS WHERE name="%s"' % process)
    if results and len(results):
      pid = results[0]
      logger.debug('Process %s believed running pid=%d .. checking' % (process,pid))
      try:
        os.kill(pid,0)
        logger.debug('Process %d is running.' % pid)
        return True
      except OSError, (errno, msg):
        logger.debug('Process not running.')

    # Need to restart process
    command = os.path.join(Variables.INSTALL_DIR,'NEOS',process+'.py')
    os.system(command)
    logger.debug('Restarted %s' % process)
      

  def checkJobs(self):
    results = self.db.queryAll('SELECT job_number,station_id,pid from QUEUE WHERE status="Running"')
    if not results or len(results) == 0:
      logger.debug('No jobs running in queue, nothing to check.')
      return

    # Wait for just-started jobs to report pid
    time.sleep(15)
    for result in results:
      jobNumber = int(result[0])
      logger.debug('Checking job #%d' % jobNumber)
      if result[1] is None or int(result[1])==0:
        logger.warning('Job %d running on unknown station.  Removing from queue')
        self.removeJob(jobNumber,'NEOS could not find where this job was running')
        continue
      else:
        station_id = int(result[1])

      if result[2] is None or int(result[2])==0:
        logger.warning('Job %d running on station id %d, but no pid registered.  Removing from queue')
        self.removeJob(jobNumber,'NEOS could not find where this job was running')
        continue
      else:
        pid = int(result[2])

      # verify job with remote solver
      stationResult = self.db.query('SELECT host_name,port from STATIONS WHERE ID=%d' % station_id)
      if stationResult and len(stationResult):
        hostname=stationResult[0]
        port = int(stationResult[1])

        remote = xmlrpclib.Server('http://%s:%d' % (hostname,port))
        msg=""
        try:
          msg = remote.verifyJob(jobNumber,pid)
        except Exception, (e):
          logger.exception(e)
          msg = "http://%s:%d not responding" % (hostname,port)
          
        if msg != 'OK':
          reason = 'Could not verify job %d running on %s: %s' % (jobNumber,hostname,msg)
          logger.debug(reason)
          logger.debug('Removing from queue.')
          self.removeJob(jobNumber,reason)
                       
        else:
          logger.debug('Job %d verified by host %s.' % (jobNumber,hostname))
        

  def removeJob(self,jobNumber,reason):
    neos = xmlrpclib.Server('http://%s:%d' % (Variables.NEOS_HOST,Variables.NEOS_PORT))
    neos.submitErrorResults(jobNumber,Variables.UBERPASSWORD,reason)

      
      
  
        
if __name__=="__main__":

  if not os.access(Variables.LOGDIR, os.X_OK):
    os.makedirs(Variables.LOGDIR)

  logger = logging.getLogger('checker')
  hdlr = logging.handlers.RotatingFileHandler(os.path.join(Variables.LOGDIR,"checker"),maxBytes=100000,backupCount=1)
  formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
  hdlr.setFormatter(formatter)
  logger.addHandler(hdlr)
  logger.setLevel(logging.DEBUG)

  checker = Checker()
  checker.checkProcess('NeosServer')
  checker.checkProcess('scheduler')
  checker.checkProcess('MailHandler')
  checker.checkJobs()





