#!/usr/bin/env python
import MySQLdb
import _mysql_exceptions
import sys

from config import Variables

try:
  db = MySQLdb.connect(host=Variables.MYSQL_HOST,
                       user=Variables.MYSQL_ADMIN_USER,
                       passwd=Variables.MYSQL_ADMIN_PASSWORD,
                       db=Variables.MYSQL_DATABASE)
except _mysql_exceptions.OperationalError, (errno, msg):
  sys.stderr.write(msg + "\n")
  sys.stderr.write("Unable to connect with the MySQL database server.\n")
  sys.stderr.write("Please make sure that all variables in the Variables.py\n")
  sys.stderr.write("file are correct, and that the database exists.\n")
  sys.exit(1)

cursor = db.cursor()

cursor.execute("show tables")
tables = [cursor.fetchall()[i][0] for i in range(0,cursor.rowcount)]

if "SOLVERS" not in tables:
  cursor.execute(
    """
    create table SOLVERS
    (
    name varchar(31) NOT NULL,
    category varchar(31) NOT NULL,
    input varchar(31) NOT NULL,
    xmldescription text NULL,
    contact varchar(255) NULL,
    password varchar(255) NULL,
    stations text NULL,
    status enum("Enabled","Disabled") NOT NULL,
    UNIQUE INDEX (category,name,input)
    )
    """)

if "QUEUE" not in tables:
  cursor.execute(
    """
    create table QUEUE
    (
    job_number int unsigned PRIMARY KEY NOT NULL,
    status enum("Waiting","Running"),
    priority enum("short","long","test"),
    station_id int unsigned NULL,
    pid int unsigned NULL
    )
    """)

if "STATIONS" not in tables:
  cursor.execute(
    """
    create table STATIONS
    (
    ID int unsigned AUTO_INCREMENT PRIMARY KEY,
    solver_cat varchar(255) NOT NULL,
    solver_name varchar(255) NOT NULL,
    solver_input varchar(255) NOT NULL,
    user varchar(255) NOT NULL,
    host_name varchar(255) NOT NULL,
    port int unsigned NULL,
    pid int unsigned NULL,
    time_limit int unsigned NULL,
    mem_limit int unsigned NULL,
    file_limit int unsigned NULL,
    filesize_limit int unsigned NULL,
    max_jobs int unsigned NULL,
    priority enum('short','long')
    )
    """)
  
    

if "JOBS" not in tables:
  cursor.execute(
    """
    create table JOBS
    (
    number int unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
    status enum("Done","Running","Waiting"),
    submit_time datetime NULL,
    start_time datetime NULL,
    end_time datetime NULL,
    user varchar(255) NULL,
    clienthost varchar(255) NULL,
    interface varchar(100) NULL,
    solver_cat varchar(255) NULL,
    solver_name varchar(255) NULL,
    solver_input varchar(255) NULL,
    password varchar(255) NULL,
    station_id int unsigned NOT NULL
    )
    """)

if "PROCS" not in tables:
  cursor.execute(
    """
    create table PROCS
    (
    name varchar(255) NULL PRIMARY KEY,
    pid int unsigned NOT NULL,
    start_time datetime NULL
    )
    """)

if "CATEGORIES" not in tables:
  cursor.execute(
    """
    create table CATEGORIES
    (
    abbreviation varchar(31) NOT NULL PRIMARY KEY,
    full_name varchar(255) NOT NULL UNIQUE
    )
    """)

if "EMAIL" not in tables:
  cursor.execute(
    """
    create table EMAIL
    (
    job_number int unsigned NOT NULL,
    address varchar(255) NOT NULL,
    message varchar(255) NULL
    )
    """)
  

if "TESTJOBS" not in tables:
  cursor.execute(
    """
    create table TESTJOBS
    (
    number int unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
    status enum("Done","Running","Waiting"),
    submit_time datetime NULL,
    start_time datetime NULL,
    end_time datetime NULL,
    user varchar(255) NULL,
    interface varchar(100) NULL,
    solver_cat varchar(255) NULL,
    solver_name varchar(255) NULL,
    solver_input varchar(255) NULL,
    password varchar(255) NULL,
    station_id int unsigned NULL
    )
    """)

if "EXAMPLES" not in tables:
  cursor.execute(
    """
    create table EXAMPLES
    (
    name varchar(255) NOT NULL,
    description varchar(255) NULL,
    solver_cat varchar(31) NOT NULL,
    solver_name varchar(31) NOT NULL,
    solver_input varchar(31) NOT NULL,
    xml varchar(255) NOT NULL,
    UNIQUE INDEX (name,solver_cat,solver_name,solver_input)
    )""")
    
