#!/usr/bin/env python
import os
import shutil
import pydoc
import NeosServer


import WebPrinter
import NeosTools
from config import Variables


replacements = {"@NEOS_WEB_SITE@":Variables.NEOS_WEB_SITE,
                "@NEOS_CGI_SITE@":Variables.NEOS_CGI_SITE,
                "@NEOS_WEB_BGCOLOR@":Variables.NEOS_WEB_BGCOLOR,
                "@NEOS_WEB_DISCLAIMER@":Variables.NEOS_WEB_DISCLAIMER,
                "@NEOS_FTP_SITE@":Variables.NEOS_FTP_SITE,
                "@NEOS_COMMENTS@":Variables.NEOS_COMMENTS,
                "@NEOS_LOGDIR@":Variables.LOGDIR,
                "@INSTALL_DIR@":Variables.INSTALL_DIR}

htmlfiles = os.listdir(os.path.join(Variables.INSTALL_DIR,'WebTemplates'))
imagefiles = os.listdir(os.path.join(Variables.INSTALL_DIR,'WebTemplates','images'))
for filename in htmlfiles:
  if filename.find('.html',-5) > 0:
    old = file(os.path.join(Variables.INSTALL_DIR,'WebTemplates',filename),'r')
    text = old.read()
    old.close()
    for r in replacements.keys():
      text = text.replace(r,replacements[r])
    new = file(os.path.join(Variables.NEOS_WWW_HOME,filename),'w')
    print "Writing " + os.path.join(Variables.NEOS_WWW_HOME,filename)
    new.write(text)
    new.close()

  if filename.find('.cgi',-4) > 0:
    old = file(os.path.join(Variables.INSTALL_DIR,'WebTemplates',filename),'r')
    text = old.read()
    old.close()
    for r in replacements.keys():
      text = text.replace(r,replacements[r])
    new = file(os.path.join(Variables.NEOS_CGI_HOME,filename),'w')
    print "Writing " + os.path.join(Variables.NEOS_WWW_HOME,filename)
    new.write(text)
    new.close()
    os.chmod(os.path.join(Variables.NEOS_CGI_HOME,filename),0755)

if not os.access(os.path.join(Variables.NEOS_WWW_HOME,'images'),os.X_OK):
  os.mkdir(os.path.join(Variables.NEOS_WWW_HOME,'images'))
for imagefile in imagefiles:
  if imagefile.find('.jpg',-4) >0 or imagefile.find('.gif',-4)>0 or imagefile.find('.png',-4)>0:
    old = os.path.join(Variables.INSTALL_DIR,'WebTemplates','images',imagefile)
    new = os.path.join(Variables.NEOS_WWW_HOME,'images',imagefile)
    print 'cp %s %s' % (old,new)
    shutil.copy(old,new)
    os.chmod(new,0644)

# create solver catalog
db = NeosTools.NeosDB()
WebPrinter.makeCatalog(db,None)
db.close()

# copy any appropriate files to web directory
for f in [os.path.join('ClientTools','NeosClient.py')]:
  old = os.path.join(Variables.INSTALL_DIR,f)
  new = os.path.join(Variables.NEOS_WWW_HOME,os.path.basename(f))
  shutil.copy(old,new)
  os.chmod(new,0644)



# Create api page from NeosServer.py
hprinter = pydoc.HTMLDoc()
apipage = file(os.path.join(Variables.NEOS_WWW_HOME,'NEOS-API.html'),'w')
apipage.write("""
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
  <meta content="text/html; charset=ISO-8859-1"
 http-equiv="content-type">
  <title>NEOS-API</title>
</head>
<body bgcolor="%s">
<a href="%s">
<IMG align=left border=0 
SRC="%s/images/globe_small.jpg" ALT="NEOS Server Home">
</a>

<h2>NEOS API</h2>
NEOS runs an XML-RPC server to communicate with clients for submitting
and retrieving jobs.&nbsp; The server can communicate with clients
written in Python, Perl, PHP, C, C++, Java, Ruby, and probably other
languages as well.&nbsp; More information on XML-RPC and writing
clients can be found at <a href="http://www.xmlrpc.com">http://www.xmlrpc.com</a>
or <a
 href="http://xmlrpc-c.sourceforge.net/xmlrpc-howto/xmlrpc-howto.html">http://xmlrpc-c.sourceforge.net/xmlrpc-howto/xmlrpc-howto.html</a>.  Any NEOS job submitted using the XML-RPC API must be formulated in the XML format required by the 
desired solver.  More information on this format can be found by following the
'XML-RPC' link on the solver's page.
<br>
<br>
<br>
The NEOS XML-RPC server is located at the URL
<code>http://%s:%d</code>.
""" % (Variables.NEOS_WEB_BGCOLOR,
       Variables.NEOS_WEB_SITE,
       Variables.NEOS_WEB_SITE,
       Variables.NEOS_HOST,
       Variables.NEOS_PORT))

infoRoutines = [
   NeosServer.NeosServer.help,
   NeosServer.NeosServer.emailHelp,
   NeosServer.NeosServer.welcome,
   NeosServer.NeosServer.version,
   NeosServer.NeosServer.ping,
   NeosServer.NeosServer.printQueue,
   NeosServer.NeosServer.getSolverTemplate,
   NeosServer.NeosServer.getXML,
   NeosServer.NeosServer.listAllSolvers,
   NeosServer.NeosServer.listCategories,
   NeosServer.NeosServer.listSolversInCategory
   ]

userRoutines = [
   NeosServer.NeosServer.submitJob,
   NeosServer.NeosServer.getJobStatus,
   NeosServer.NeosServer.killJob,
   NeosServer.NeosServer.getFinalResults,
   NeosServer.NeosServer.getIntermediateResults,
   NeosServer.NeosServer.getFinalResultsNonBlocking,
   NeosServer.NeosServer.getIntermediateResultsNonBlocking
   ]

solverRoutines = [
   NeosServer.NeosServer.pingHost,
  NeosServer.NeosServer.validateSolverXML,
  NeosServer.NeosServer.registerSolver,
  NeosServer.NeosServer.removeSolver,
  NeosServer.NeosServer.disableSolver,
  NeosServer.NeosServer.enableSolver,
  NeosServer.NeosServer.registerExample,
  NeosServer.NeosServer.removeExample
  ]

apipage.write('<hr><h2>Routines for getting information from NEOS</h2>\n')
for routine in infoRoutines:
  apipage.write(hprinter.docroutine(routine,cl=NeosServer.NeosServer).replace('(self)','()').replace('(self, ','('))

apipage.write('<hr><h2>Routines for submitting and retrieving jobs on NEOS</h2>\n')

for routine in userRoutines:
  apipage.write(hprinter.docroutine(routine,cl=NeosServer.NeosServer).replace('(self)','()').replace('(self, ','('))

apipage.write('<hr><h2>Routines for maintaining solvers on NEOS</h2>\n')

for routine in solverRoutines:
  apipage.write(hprinter.docroutine(routine,cl=NeosServer.NeosServer).replace('(self)','()').replace('(self, ','('))

apipage.write("""<hr>
A Python client is included in the NEOS download package as <a
 href="./NeosClient.py">NeosClient.py</a>:<br>
<code>#!/usr/bin/env python<br>
import sys<br>
import xmlrpclib<br>
import time<br>
<br>
from config import Variables<br>
<br>
if len(sys.argv) &lt; 2 or len(sys.argv) &gt; 3:<br>
&nbsp; sys.stderr.write("Usage: NeosClient &lt;xmlfilename | help |
queue&gt;\n")<br>
&nbsp; sys.exit(1)<br>
<br>
neos=xmlrpclib.Server("http://%s:%d" % (Variables.NEOS_HOST,
Variables.NEOS_PORT))<br>
<br>
if sys.argv[1] == "help":<br>
&nbsp; sys.stdout.write("Help not yet available...\n")<br>
<br>
elif sys.argv[1] == "queue":<br>
&nbsp; msg = neos.printQueue()<br>
&nbsp; sys.stdout.write(msg)<br>
<br>
else:<br>
&nbsp; xmlfile = open(sys.argv[1],"r")<br>
&nbsp; xml=""<br>
&nbsp; buffer=1<br>
&nbsp; while buffer:<br>
&nbsp;&nbsp;&nbsp; buffer =&nbsp; xmlfile.read()<br>
&nbsp;&nbsp;&nbsp; xml+= buffer<br>
&nbsp; xmlfile.close()<br>
<br>
&nbsp; (jobNumber,password) = neos.submitJob(xml)<br>
&nbsp; sys.stdout.write("jobNumber = %d\n" % jobNumber)<br>
<br>
&nbsp; offset=0<br>
&nbsp; while status != "Done":<br>
&nbsp;&nbsp;&nbsp; (msg,offset) =
neos.getIntermediateResults(jobNumber,password,offset)<br>
&nbsp;&nbsp;&nbsp; sys.stdout.write(msg.data)<br>
&nbsp;&nbsp;&nbsp; status = neos.getJobStatus(jobNumber, password)<br>
<br>
&nbsp; msg = neos.getFinalResults(jobNumber, password).data<br>
&nbsp; sys.stdout.write(msg)</code>
<br>
""")
apipage.write("<p>\n%s\n</body>\n</html>\n" % Variables.NEOS_WEB_DISCLAIMER)
apipage.close()
              
