#!/usr/bin/env python

# AMPL.Environment
import sys
import re
import popen2
import os
import socket
import StringIO

import SolverTools.SolverBase
from config import Variables


class AMPLSolver(SolverTools.SolverBase.SolverBase):
  def __init__(self,solver):
    SolverTools.SolverBase.SolverBase.__init__(self)
    self.solvername=solver
    
    # Determine location for scratch directory
    host = socket.gethostbyaddr(socket.gethostname())[0]
    if re.search(r'neos\.mcs\.anl\.gov',host) or \
       re.search(r'vieta\.mcs\.anl\.gov',host):
      tempdir='/disks/tmp/neos-' + str(os.getpid())
    elif re.search(r'mcs.anl.gov',host):
      tempdir='/sandbox/neos/neos-' + str(os.getpid())
    elif re.search(r'iems.nwu.edu',host) or \
         re.search(r'iems.northwestern.edu',host):
      tempdir='/home/tate/neos/tmp/neos-' + str(os.getpid())
    else:
      import tempfile
      tempdir = os.path.join(tempfile.gettempdir(),'neos-'+str(os.getpid()))
    os.environ['TMPDIR'] = tempdir
    try:
      os.makedirs(tempdir)
    except OSError, (errno, msg):
      print "Could not create " + tempdir

    self.setScratchDir(tempdir)


    

  def execute(self):
    sys.stdout.write("You are using the solver " + self.solvername + ".\n")
    self.printComments()


    modelfile = None
    datafile = None
    commandsfile = None
    if os.access("ampl.mod",os.R_OK):
      modelfile = open("ampl.mod","r")

    if os.access("ampl.dat",os.R_OK):
      datafile = open("ampl.dat","r")

    if os.access("ampl.com",os.R_OK):
      commandsfile = open("ampl.com","r")


    sys.stdout.write("Executing AMPL.\n")
    amplin = ""
#   Set the solver and options before loading the data.
#   People whose model includes a solve statement need this set first

    amplin += "option show_stats 1;\n"
    amplin += 'option solver "%s";\n' % self.solvername
      
    if modelfile:
      amplin += "model;\n"
      for line in modelfile.readlines():
        amplin += line
      modelfile.close()
    else:
      sys.stderr.write("MODEL file not found\n")

    if datafile:
      amplin += "data;\n"
      for line in datafile.readlines():
        amplin += line
      datafile.close()

    solvefound = None
    if commandsfile:
      for line in commandsfile.readlines():
        amplin += line
        if re.search(r'solve |solve;',line,re.IGNORECASE):
          solvefound = 1
      commandsfile.close()
    if not solvefound:
      amplin += "solve;\n"

    sys.stdout.write(amplin)
    SolverTools.SolverBase.SolverBase.execute(self,Variables.AMPL, options="-R", instream=amplin)
    
    self.rmScratchDir()


if __name__ == "__main__":
  solver = AMPL.AMPLSolver(argv[1])
  solver.execute()
