import os
import sys
import re
import socket


import SolverTools.SolverBase
from config import Variables

class GAMSSolver(SolverTools.SolverBase.SolverBase):
  def __init__(self,solver):
    SolverTools.SolverBase.SolverBase.__init__(self)
    self.options = GamsOptions()
    self.solvername = solver

  def checkForOptions(self):
    if os.access(self.solvername+".opt",os.R_OK):
      self.options['optfile']='1'
      
  def checkForModel(self):
    sys.stdout.write("Checking for model file.\n")
    try:
      model_gams = open("MODEL.gms","w+")
    except IOError, (errno, strerror):
      sys.stderr.write("IO error(%s): %s\n" % (errno, strerror))
      sys.stderr.write("Cannot open MODEL.gms for writing\n")
      sys.stdout.write("Cannot open MODEL.gms for writing\n")
      sys.exit(1)

    
    model_gams.write("$offlisting\n$offsymlist\n$offsymxref\n$offuellist\n")

    try:
      modelfile = open("gams.mod","r")
      for line in modelfile.readlines():
        model_gams.write(line)
    except IOError, (errno, strerror):
      sys.stderr.write("IO error(%s): %s\n" % (errno, strerror))
      sys.stderr.write("Cannot find or open file gams.mod\n")
      sys.stdout.write("Cannot find or open file gams.mod\n")
      sys.exit(1)

        
    self.model = "MODEL.gms"


  def printOutput(self):
    sys.stdout.write("Composing results.\n")
    if os.access("solve.out",os.R_OK):
      try:
        sys.stdout.write("\n%% GAMS OUTPUT %%%%%%%%%%%%%%%%%%%%%%%%%\n")
        f = open("solve.out","r")
        for line in f.readlines():
          sys.stdout.write(re.sub(r' (\/\S*\/)*',' ',line))
        f.close()
        sys.stdout.write("\n\n")
      except IOError:
        sys.stdout.write("Could not open solve.out\n")
    else:
      sys.stdout.write("solve.out is not readable/doesn't exist\n")

    sys.stdout.flush()

    if os.access("WANTLOG",os.F_OK):
      try:
        file = open("gams.logfile","r")
        sys.stdout.write("\n%% GAMS LOGFILE %%%%%%%%%%%%%%%%%%%%%%%%\n")

        for line in file.readlines():
          sys.stdout.write(re.sub(r' (\/S*\/)*',' ',line))
        file.close()
        sys.stdout.write("\n\n")
      except IOError:
        sys.stdout.write("Could not open gams.logfile\n")
      sys.stdout.flush()

  def execute(self):
    self.printComments()
    self.checkForModel()
    self.checkForOptions()
    
    sys.stdout.write("Calling GAMS.\n\n")
    sys.stdout.flush()

    # Determine location for scratch directory
    host = socket.gethostbyaddr(socket.gethostname())[0]
    if re.search(r'neos\.mcs\.anl\.gov',host) or \
       re.search(r'vieta\.mcs\.anl\.gov',host):
      tempdir='/disks/tmp/neos-' + str(os.getpid())
    elif re.search(r'mcs.anl.gov',host):
      tempdir='/sandbox/neos-' + str(os.getpid())
    elif re.search(r'iems.nwu.edu',host) or \
         re.search(r'iems.northwestern.edu',host):
      tempdir='/home/tate/neos/tmp/neos-' + str(os.getpid())
    

    # Create scratch dir for gams
    os.mkdir(tempdir)
    self.options['scrdir']=tempdir
    self.setScratchDir(tempdir)

    try:
      # Call parent execute command
      SolverTools.SolverBase.SolverBase.execute(self,Variables.GAMS,
                                                options=str(self.options))
      self.printOutput()
      self.rmScratchDir()
    except Exception, e:
      raise Exception, e
    

class GamsOptions:
  def __init__(self):
    self.dict = {'ps':'0',       'pw':'79',
                 'execmode':'322','limrow':'0',  'limcol':'0',
                 'relpath':'1',   'suppress':'0','o':'solve.out',
                 'lo':'2',        'lf':'gams.logfile',
                 'al':'0',        'ao':'0',      'cerr':'10'}
    self.model = 'gams.mod'

  def __str__(self):
    s = " " + self.model
    for k in self.dict.keys():
      s += " -%s=%s" % (k, self.dict[k])

    return s

  def __setitem__(self,key,value):
    self.dict[key]=value;

  def __getitem__(self,key):
    return self.dict[key]


  def update(self,d):
    self.dict.update(d)
