
import os
import sys
import popen2
import re
import select
import signal

class SolverBase:
  def __init__(self):
    self._tempdir = None
    
  def __del__(self):
    pass

  def whatami(self):
    """
    returns an array of [cat,solver,input]
    """
    f = file('neos-solver','r')
    return f.read().strip().split(':')
  
  def securityCheck(self,filename):

    # Check for #includes in source and object files
    files_to_check = [filename]
    if re.search(r'\.o$',filename):
      for f in os.listdir(os.path.dirname(os.path.abspath(filename))):
        if os.path.isfile(f) and re.match(re.sub(r'\.o',r'',filename), f):
          files_to_check.append(f)

    include = re.compile(r"""#include\s*('|")""")   #"fix typefacing
    for f in files_to_check:
      file = open(f)
      for line in file.readlines():
        if include.search(line):
          file.close()
          raise UnsafeFile("For security reasons, you may not #include generic files." + f)

    # Check for bad symbols in file
    bad = re.compile(r'creat|unlink|fprint|open|socket|pipe|fork|exec|kill|signal|longjmp|main|system|fscanf|_f_open|_system_|_do_l_out')
    pipe = os.popen("nm " + filename)
    for line in pipe.readlines():
      if bad.search(line):
        file.close()
        raise UnsafeFile('Read, write, or system commands are not allowed in ' + filename)
      
    

  def execute(self,command,instream=None,options=None,optionsDict=None,
              outfile=None,errfile=None):
    if command == "":
      sys.stderr.write("Error in solve script: command not set")
      sys.exit(1)

    ops = ""
    if options is not None:
      ops += " %s" % options
    if optionsDict is not None:
      ops += str(optionsDict)
    sys.stdout.write("nice -15 %s %s\n" % (command, ops) )
    pipe = popen2.Popen3("nice -15 %s %s" % (command, ops),1 )
    input = pipe.tochild
    output = pipe.fromchild
    error = pipe.childerr

    if outfile is None:
      outfile = sys.stdout
    if errfile is None:
      errfile = sys.stderr
      
    if instream is not None:
      input.write(instream)
    input.close()

    outputClosed=0
    errorClosed=0
    msg = 1
    while not (outputClosed and errorClosed):
      try:
        ready = select.select([output, error], [], [])
        if len(ready[0]):
          if error in ready[0]:
            msg = error.readline()
            if msg:
              errfile.write(msg)
              errfile.flush()
            else:
              errorClosed = 1
          if output in ready[0]:
            msg = output.readline()
            if msg:
              outfile.write(msg)
              outfile.flush()
            else:
              outputClosed = 1
        if outputClosed and errorClosed:
          self.stop=1
      except IOError, (err):
        errfile.write("Job died unexpectedly")
    output.close()
    error.close()

    # Check return value from command
    status = 1
    try:
      status = pipe.wait()
    except Exception, (e):
      outfile.write(str(e))
    if status != 0:
      outfile.write(" Error (" + str(status) + ") in " + command + "\n")
    return status


  def setScratchDir(self, dir):
    """Set a directory that will be deleted when the process recieves SIGTERM.
    To prevent accidents, the directory name must end with the process id."""
    if re.search("%s$" % str(os.getpid()), dir):
      self._tempdir = dir
      try:
        os.mkdir(dir)
      except OSError, (errno, msg):
        print msg
      signal.signal(signal.SIGTERM, self.sigtermHandler)

    else:
      sys.stderr.write("Scratch directory must end in job Number.\n")
      sys.stderr.write(dir + "\n")
    

  def rmScratchDir(self):
    if self._tempdir:
      try:
	files = os.listdir(self._tempdir)
	for f in files:
	  os.remove(os.path.join(self._tempdir,f))
        os.rmdir(self._tempdir)
      except IOError, (errno, msg):
	print msg
      
  def sigtermHandler(self,signo,frame):

    self.rmScratchDir()
    sys.exit(signo)
    
  def printComments(self):
    commentfile = None
    for commentfilename in ("comment","comments","COMMENT","COMMENTS"):
      if commentfile is None and os.access(commentfilename,os.R_OK):
        commentfile = open(commentfilename,"r")
      break

    if commentfile is not None:
      sys.stdout.write("\n%% YOUR COMMENTS %%%%%%%%%%%%%%%%%%%%%%%\n")
      for line in commentfile.readlines():
        sys.stdout.write(line)
      sys.stdout.write("\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n")


    
class OptionsList:
  def __init__(self):
    self.dict = {}

  def __str__(self):
    keys = ""
    for k in self.dict.keys():
      if self.dict[k] is None:
        keys += " %s" % k
      else:
        keys += " %s %s" % (k,str(self.dict[k]))
    keys += " "
    return keys

  
  def update(self, d):
    self.dict.update(d)
  
  def __getitem__(self,key):
    return self.dict[key]

  def __setitem__(self,key,value):
    self.dict[key] = value
    

class UnsafeFile(Exception):
  def __init__(self, value):
    self.value = value
  def __str__(self):
    return repr(self.value)

if __name__=="__main__":
  solver = SolverBase(34)
  solver.execute("ls")
